(* :Title: TuGames.m *)
Off[Syntax::stresc]
(* :Context: TuGames` *)

(* :Summary:	
    This package is an extension to the package written by Michael Carter for 
    modeling and calculating solutions and properties for cooperative games with 
    transferable utilities. 
*)			

(* :Author:		
    Holger Ingmar Meinhardt
    Department of Economics
    University of Karlsruhe 
    hme@vwl3.wiwi.uni-karlsruhe.de			
*)

(* :Package Version: 1.1 *)

(* 
   :Mathematica Version: 3.0, 4.0, 4.1, 4.2, 5.0, 5.1 

   It seems that functions that are using ConstrainedMax[] or
   ConstrainedMin[] need a code revision for
   Mathematica Version > 5.0. These functions
   will be deprecated in future Mathematica versions.
   To resolve this problem one should replace the ConstrainedMax[]
   and ConstrainedMin[] commands by Maximize[] and Minimize[], respectively.
   Do the same in the package Cooperat.m written by Michael Carter.
   A negative side effect associated with this command replacement is a 
   considerable increase of calculation time. Since the Mathematica version 
   5.0 was recently released, we have up to now no knowledge if the code 
   modification cause any stability problems.

*)

(* :Package: Tested under Linux x86/64, HP-UX and AIX *)

(*:Keywords:
   Dual Game, Superadditive Game, Convex Game, Strong Convex Game, Average-Convex Game,
   Kernel, balancing Maximal Excesses.
*)

(* :Sources:
   
   Chih Chang and Theo Driessen, (Pre)Kernel Catchers for Cooperative Games,
   OR Spectrum, Vol. 17, 1995

   Chih Chang and Ching Yu Kan, The Bound of the Kernel, Mathematical Social
   Sciences, Vol. 25, 87-93, 1992.

   Chih Chang and Chrong-Hisan Lian, Some Results on (Pre)Kernel Catchers and
   the Coincidence of the Kernel and Prekernel, International Game Theory Review,
   Vol. 4, No. 3, 201-211, 2002. 

   Theo Driessen, Cooperative Games, Solutions and Applications, Kluwer Academic
   Publishers, Dordrecht, 1988.

   E. Inarra and J. Usategui, The Shapley value and average convex games, 
   IJGT, 22, 13-29, 1993.

   U. Faigle, W. Kern and J. Kuipers, An efficient algorithm for nucleolus and
   prekernel computation in some classes of Tu Games. Memorandum No. 1464,
   Faculty of Mathematical Sciences, University of Twente, 1998.

   Y. Funaki, Upper and Lower Bounds of the Kernel and Nucleolus, 
   IJGT, 121-129, 1986.

   M. Maschler, The Bargaining Set, Kernel and Nucleolus, Handbook of Game
   Theory, Chapter 18, 591-647, 1992.

   M. Maschler, B. Peleg and L.S. Shapley, Geometric Properties of Kernel,  
   Nucleolus and related Concepts, in Mathematics of Operations Research, 
   Vol4 Nov. 1979, p. 303-338.

   H. Meinhardt, An LP approach to compute the pre-kernel for cooperative games, 
   Computers and Operation Research, Vol 33/2 pp. 535-557,2005. 

   C. Rafels and N. Ybern, Even and Odd Marginal Worth Vectors, Owen's
   Multilinear Extension and Convex Games, IJGT, 113-126, 1995.

   R.E. Stearns, Convergent Transfer Schemes for N-Person Games,
   Transaction American Mathematical Society, 449-459, 1968.

   Hal Varian (Ed.). Economics and Financial Modeling with Mathematica, 
   Springer, 1992. 
*)


(* 
   :History:
   Version 1.0 without options
   Version 1.1 with options
*)


(* 
    :Limitations: 
    see ManualTuGames.nb or ManualTuGames.pdf  
*)


(* 
   :General Remarks:

   You may modify your copy or copies of the package or any portion
   of it. To let profit other users from your modifications forward 
   these as well as any bug reports to the address mentioned above. 
   By forwarding your modifications please keep the convention to list 
   first the old code portion and then the part you have modified 
   with a brief explanation.  

*)



(*:Requirements: 
 
   The Package Cooperat.m by Michael Carter and VertexEnum.m 
   by Komei Fukuda and Ichiro Mizukoshi. The program 'cddmathlink' by Komei
   Fukuda is highly recommended for calculating all extreme
   points of the core. Nevertheless, it is not necessary to have installed
   the C-library on your computer to run properly the package 'TuGames.m'. 
   The C-library is only available for Unix and can be found under
   http://www.ifor.math.ethz.ch/staff/fukuda/fukuda.html 
   As a substitute use the function VerticesCore[] that is based on
   VertexEnum.m. 
*)



BeginPackage["TuGames`"]
Print["==================================================="];
Print["Loading Package 'TuGames' for ", $OperatingSystem];
Print["==================================================="];
Print["TuGames V1.1 by Holger I. Meinhardt"];
Print["Release Date: 27.07.2005"];
Print["Program runs under Mathematica Version 3.0 or later"];
(*
welc=NotebookOpen["/home/holger/.Mathematica/Applications/TuWelcome.nb"];
Pause[1];
NotebookClose[welc];
*)
Print["==================================================="];

(* 
The following describes some useful functions in the package to
check properties of the game.
*)


DefineGame::usage = 
"DefineGame[T,values] defines the game which we want to analyze. \n
 T is the player set and values are the worth for the coalitions.";


DualGame::usage= 
"DualGame[game] gives the value for all coalitions what they \n
 can extract from the grand coalition by leaving the grand coalition.";

SuperAdditiveQ::usage = 
"SuperAdditiveQ[game] checks if a game is superadditive.";

WeaklySuperAdditiveQ::usage = 
"WeaklySuperAdditiveQ[game] checks if a game is weakly superadditive. \n
 Suppose the coalitions S_1,S_2,...S_k partition the grand coalition T, \n 
 then the game v is weakly superadditive, if sum_i v(S_i) <= v(T). \n
 This superadditivity property is a necessary condition for the core \n 
 of the game v to be nonempty.";

Assign::usage = 
"Assign[values] assigns the values of the game. "

SelectSuperSets::usage = 
"SelectSuperSets[i] selects all supersets of i"
 
IncreasingMargContributions::usage = 
"IncreasingMargContributions[game,i] checks if the marginal contribution \n
 of player i is increasing.";


ConvCheck::usage = 
"ConvCheck[game] checks the convexity of the game. \n
 It provides information whether the marginal contributions \n
 are increasing for each player.";

ConvexQ::usage = 
"ConvexQ[game] checks if the Tu-game is convex. It returns the value 'True' or 'False'.";

StrIncreasMargContrib::usage = 
"StrIncreasMargContrib[game,i] checks if the marginal contribution \n
 of player i is strictly increasing."; 
  
ConvStrQ::usage = 
"ConvStrQ[game] checks if the Tu-game is strictly convex. \n
 It provides information whether the marginal contributions are increasing \n
 for each player.";

ConvexStrQ::usage = 
"ConvexStrQ[game] checks if the Tu-game is strictly convex. \n
 It returns only 'True' or 'False'.";

AvConvexQ::usage = 
"AvConvexQ[game] checks the average-convexity of the game. \n
 It returns 'True' or 'False'.";

AvSumCheck::usage = 
"AvSumCheck[game, S, liste] calculates if the average sum of \n 
 marginal contributions is increasing.";

GameMonotoneQ::usage = 
"GameMonotoneQ[game] verifies if the game is monotone.";

MonotoneQ::usage = 
"MonotoneQ[game] verifies if the game is monotone.";

ZeroMonotoneQ::usage = 
"ZeroMonotoneQ[game] verifies if the game is zero-monotone.";

ZeroNormalization::usage = 
"ZeroNormalization[game] determines the zero-normalized game.";

ZeroOneNormalization::usage = 
"ZeroOneNormalization[game] determines the (0,1)-normalized game.";

OneNormalization::usage = 
"OneNormalization[game] determines the one-normalized game.";

ContributionSum::usage = 
"ContributionSum[game,R,S] or ContributionSum[R,S]"

SmallestContributionVector::usage = 
"SmallestContribution[game] calculates the vector of the smallest contributions.";

SmallestContribution::usage = 
"SamllestContribution[game,i] calculates the smallest contribution of player i.";

Scrb::usage = 
"Scrb[game,i] calculates the separable cost-remaining benefits for player i.\n
See Funaki (1986).";

ScrbSolution::usage = 
"ScrbSolution[game] calculates the separable cost-remaining benefits.\n
See Funaki (1986).";

TIJsets::usage = 
"TIJsets[i,j] computes the sets which contains player i but not j.";


ExcessPayoff::usage = 
"ExcessPayoff[game,payoff] calculates the excess vector for 'payoff'.";

MaxExcessBalanced::usage = 
"MaxExcessBalanced[game,payoff] determines whether the maximum \n
 excesses induced by the payoff vector/vectors are maximal balanced \n 
 among the players. Be careful, the function does not check the \n
 efficiency property.  For checking in addition the efficiency \n
 property consult the function Pre-Kernel[]."; 

MaxExcessSets::usage = 
"MaxExcessSets[game,payoff] computes the set of proper coalitions
 with largest excess."

IntersectionOfMaxExcessSets::usage = 
"IntersecionOfMaxExcessSets[game, payoff] determines if the set
 of proper coalitions with maximal excess has an empty intersection."

KernelCalculation::usage = 
"KernelCalculation[game,options] computes Kernel or Pre-Kernel \n
 points of the game. Optional parameters are: \n
 ChangeInternalEps, DisplayAllResults, EpsilonValue and SetGameToNonZeroMonotonic.\n
 The first optional parameter AVChangeInternalEps can be used to speed up \n 
 computation or to search for different allocations in the initial LPs. \n
 If DisplayAllResults is set to 'True', then the return value is related \n
 to a Kernel solution, objective function, constraint set of the final LP, \n 
 all bi-symmetrical transfers and the allocations computed by the initial LPs.\n
 In this case invoke the function by {solker,obj,const,tra,pay}=KernelCalculation[game]. \n
 Default value is false, that means that the returnvalue is just related to
 a Kernel Solution and the allocation obtained by the initial LPs. \n  
 EpsilonValue is a critical value to change the strong epsilon core and \n 
 it can be calculated, for instance, by the function FirstCriticalVal[game].\n
 Solution can be checked with the functions KernelImputationQ, KernelImputationListQ, \n 
 PreKernelQ, or MaxExcessBalanced. To search for Kernel solution for \n
 non zero-monotonic games set the last variable to 'True', but in this case \n
 the function need not to terminate. For its default value 'False' the function \n
 Kernel[game] will be invoked to avoid infinite loops.";

Kernel::usage = 
"Kernel[game,options] computes a kernel point of the game from one LP. \n 
 The optional parameter are DisplayAllResults and EpsilonValue. If \n 
 DisplayAllResults is set to 'True', then the return value is related \n
 to a Kernel solution, objective function, constraint set of the final LP, \n 
 the complete variable set and all bi-symmetrical transfers. \n
 In this case invoke the function by {solker,obj,const,var,tra}=Kernel[game]. \n
 EpsilonValue is a critical value to change the strong epsilon core and \n 
 it can be calculated, for instance, by the function FirstCriticalVal[game].";

KernelVertices::usage =
"KernelVertices[game,options] computes the vertices of the Kernel solution
 from an LP."

ModifiedKernel::usage = 
"ModifiedKernel[game,options] computes a kernel point from the least core. \n
 That means, a vertex of the kernel solution inside of the least core will be computed. \n
 Thus, in many cases the Nucleolus should be computed. The algorithm is based \n
 on a method by Peleg to translate the definition of the Nucleolus into \n
 a sequence of linear programs. Note that the function might not compute \n
 the correct value for inessential games. Check the result with \n
 KernelImputationListQ[]."


ModifiedNucleolus::usage = 
"ModifiedNucleolus[game,options] computes the nucleolus from the least core. \n
 The algorithm is based on a method by Peleg to translate the definition of the \n
 Nucleolus into a sequence of linear programs. The recursion stops, if the set \n
 of new equal constraints is empty. Note that the function might not compute \n
 the correct value for inessential games. Check the result with \n
 KernelImputationListQ[]."

LexiCenter::usage = 
"LexiCenter[game,options] computes the lexicographic center of a TU-game, i.e. \n
 the nucleolus. Note that the function might not compute \n
 the correct value for inessential games. Check the result with \n
 KernelImputationListQ[]."

PreNucleolus::usage = 
"PreNucleolus[game,options] computes the pre-nucleolus from the set of pre-imputations. \n
 The algorithm is based on a method by Peleg to translate the definition of the Nucleolus into \n
 a sequence of linear programs on the pre-imputation set. Note that the function might not compute \n
 the correct value for inessential games. Check the result with \n
 KernelImputationListQ[]."


FindKernelSolution::usage = 
"FindKernelSolution[game,payoff] try to approximate a pre-kernel solution by iteratively \n 
carrying out transfers between pairs of players. The suggest algorithm is due to \n
M. Maschler. For details see U. Faigle, W. Kern and J. Kuipers (1998)."


FindPreKernelSolution::usage = 
"FindKernelSolution[game,payoff] try to approximate a pre-kernel solution by iteratively \n 
carrying out transfers between pairs of players. The suggest algorithm is due to \n
M. Maschler. For details see U. Faigle, W. Kern and J. Kuipers (1998)."


KernelImputationQ::usage = 
"KernelImputationQ[game,payoff] checks whether the payoff belongs to the Kernel.";

KernelImputationListQ::usage = 
"KernelImputationListQ[game,payoff] checks whether the list of payoffs belongs to the Kernel.";
                                
PreKernelQ::usage = 
"PreKernelQ[game,payoff] checks whether the payoff belongs \n
 to the Pre-Kernel. PreKernelQ checks also the efficiency condition \n
 in contrast to the function MaxExcessBalanced.";
                            
Transfer::usage = 
"Transfer[payoff, T, d, i, j] determines the bi-symmetrical transfer \n 
 with respect to the strong epsilon-core. The function Transfer[] \n
 computes the endpoint of the imputation 'payoff' in the i-j \n
 direction by transferring the amount d_ij from player i to player j. \n
 The argument payoff is the initial payoff distribution, \n
 T is the player set, d is the amount which should be \n
 transferred from player i to j.";

DeltaLP::usage = 
"DeltaLP[game, i, j, eps] determines the largest amount d_ij that can be \n
 transferred from player i to player j while remaining in the epsilon-core.";

TransferConstraints::usage = 
"TransferConstraints[game, i, j] or TransferConstraints[game, i, j, eps] \n 
 computes the constraints for calculation the largest amount that be transferred from \n
 player i to j while remaining in the core or strong epsilon core. \n
 Default value for eps is zero and the value eps can be omitted \n
 by calling the function.";

AllConstraints::usage = 
"AllConstraints[game] computes the constraints set and variable \n
 set that are needed to calculate a kernel point of the game.";

AllMaxSurpluses::usage = 
"AllMaxSurpluses[game,payoff] calculates the maximal surplus of player i \n
 over j with respect to the imputation 'payoff' for all possible \n
 player combinations. Note that the efficiency condition will not \n
 be checked."

MaxSurplus::usage = 
"MaxSurplus[game,pi,pj,payoff] calculates the maximal surplus of player i \n
 over j with respect to the imputation 'payoff'. Note that the efficiency \n
 condition will not be checked."


EpsValue::usage = 
"EpsValue[game,eps] assigns the amount eps to every proper coalition.";
                    
EpsCore::usage = 
"EpsCore[game] determines the least-core. Gives more information than \n
 the function LeastCore[game].";

FirstCriticalVal::usage  = 
"FirstCriticalVal[game] calculates the smallest epsilon such that the \n
 epsilon-core is nonempty.";

SecondCriticalVal::usage = 
"SecCriticalVal[game] calculates the biggest gain that any group of \n
 individuals can assure by forming a coalition. The epsilon core is \n 
 included in the imputation set if and only if epsilon is greater than \n
 the sec-critical-val. See Maschler, Peleg and Shapley (1979) on \n
 page 306 for more details.";
                         
ThirdCriticalVal::usage = 
"ThridCriticalVal[game] calculates another critical value for epsilon to generalize the
 bisection property to strong epsilon core. The epsilon core is included in the reasonable 
 set if and only if epsilon greater or equal than the third critical value.";
                            

FourthCriticalVal::usage = 
"FourthCriticalVal[game] returns the minimum of the SecCriticalVal and \n
 the ThirdCrititcalVal.";

FifthCriticalVal::usage = 
"The Union of the Lower set and the Core is included in the strong \n
 epsilon core if and only if epsilon >= epsilon5. See Chang and Kan (1992) \n
 for more details."

StarCriticalVal::usage = 
"StarCriticalVal[game] determines a bound for epsilon as defined \n
 by Maschler, Peleg and Shapley (1979) on page 320. The epsilon-core \n
 is included in the intersection of the imputation set and \n
 the reasonable set if and only if epsilon is greater or equal \n
 than this epsilon bound. Hence, the kernel is included in the
 strong epsilon core if epsilon >= epsilon*." 

SecondStarCriticalVal::usage = 
"SecondStarCriticalVal[game] determines a bound for epsilon as defined \n
 by Chang and Kan (1992) on page 92. This is a better lower \n
 epsilon bound than the epsilon bound obtained by StarCriticalVal[].\n
 The kernel is included in the strong epsilon core if epsilon >=
 epsilon**." 

ThirdStarCriticalVal::usage = 
"ThirdStarCriticalVal[game] determines a bound for epsilon as defined \n
 by Chang and Driessen (1994). This is a better lower  \n
 epsilon bound than the epsilon or epsilon** bound obtained by StarCriticalVal[]. \n
 and ThirdStarCritical. The kernel is included in the strong epsilon core if \n
 epsilon >= epsilon***." 

LowerSetQ::usage = 
"LowerSetQ[game] checks if the lower set is non empty. An important set
 to define kernel catcher."

UpperSetQ::usage = 
"UpperSetQ[game] checks if the upper set is non empty. An important set
 to define kernel catcher. Furthermore, the kernel is equal to the pre-kernel."

LowerSetIncImputationQ::usage = 
"LowerSetIncImputationQ[game] checks if the lower set is included in the
imputation set. This is equivalent to the statement that the game is zero-monotone."

UpperSetIncImputationQ::usage = 
"UpperSetIncImputationQ[game] checks if the upper set is included in the
imputation set."


IntersectionUpperLowerSetQ::usage = 
"IntersectionUpperLowerSetQ[game] checks if the intersection of the lower \n
 and upper set is non-empty."

GrandCoalitionLargestValueQ::usage = 
"GrandCoalitionLargestValueQ[game] checks if the worth of the grand coalition \n
 is the largest."


PreKernelEqualsKernelQ::usage = 
"PreKernelEqualsKernelQ[game] checks the classes of games where the prekernel \n
 is equal to the kernel."

LargestAmount::usage = 
"LargestAmount[game] or LargestAmount[game,i] indicates the largest amount \n
 a player i can contribute to a coalition.";

ProperAmount::usage = 
"ProperAmount[game_] or ProperAmount[game,i] indicates the largest amount \n
 a player i can contribute to a proper coalition.";
                        
ReasonableSet::usage = 
"ReasonableSet[game] returns the set of imputations that give no player more \n
 than his largest amount that he can contribute to a coalition.";
                        
ReasonableOutcome::usage = 
"ReasonableOutcome[game] set the largest amount to the variable r.";

Quota::usage = "Quota[game] determines the quotas of the game.";

VerticesCore::usage = 
"VerticesCore[game] calculates all vertices of the core. \n
 Be careful for games greater than n > 6. Calculation time explodes.";

CddVerticesCore::usage = 
"CddVerticesCore[game] calculates all vertices of the core with Cddmathlink via Mathlink.
Cddmathlink uses floating point arithmetic.";

(*
CddGmpVerticesCore::usage = 
"CddGmpVerticesCore[game,Options] calculates all vertices of the core with Cddmathlink2gmp via Mathlink.
Cddmathlink2gmp uses GMP rational exact arithmetic.";

CddGmpPlotCore::usage = 
"CddGmpPlotCore[game,Options] plot the core of a game with Cddmathlink2gmp via Mathlink.
Cddmathlink2gmp uses GMP rational exact arithmetic.";
 
CddGmpPlotImputationSet::usage = 
"CddGmpPlotCore[game,Options] plot the core of a game with Cddmathlink2gmp via Mathlink.
Cddmathlink2gmp uses GMP rational exact arithmetic.";

CddGmpPlotReasonableSet::usage = 
"CddGmpPlotReasonableSet[game,Options] plot the reasonable set of a game with Cddmathlink2gmp via Mathlink.
Cddmathlink2gmp uses GMP rational exact arithmetic.";
*)


CddVerticesImputationSet::usage = 
"CddVerticesReasonableSet[game] calculates all vertices of the \n
 imputation set with Cddmathlink via Mathlink."; 
                          
CddVerticesReasonableSet::usage = 
"CddVerticesReasonableSet[game] calculates all vertices of the \n 
 reasonable set with Cddmathlink via Mathlink.";

CddVerticesLowerSet::usage = 
"CddVerticesLowerSet[game] calculates all vertices of the \n 
 lower set with Cddmathlink via Mathlink.";

(*
CddGmpPlotLowerSet::usage = 
"CddGmpPlotLowerSet[game,Options] plot the lower set of a game with Cddmathlink2gmp via Mathlink.
Cddmathlink2gmp uses GMP rational exact arithmetic.";

FilledCore::usage =
"FilledCore[game,payoff,opts] plots the core of a three person game. \n
 Fill the core area with gray color.";

*)
Vec3DToSimplex::usage = 
"Vec3DToSimplex[{x1,x2,x3}] transforms a vector of dimension 3 into a vector of dimension 2."

(*
AllVertices::usage = 
"AllVertices[m,d1,Flatten[extmat]]. Call of the C-library to \n
 determine all vertices of a convex polyhedron. See documentation Fukuda.";
*)

Nuc1convex::usage = 
"Nuc1convex[game] can be only used to calculate the nucleolus for 1-convex games.";

NewShapley::usage = 
"NewShapley[game] calculates the Shapley Value of the game.";
		    
UtopiaVector::usage = 
"UtopiaVector[game] calculates the utopia payoff for all players.";  

UtopiaPayoff::usage = 
"UtopiaPayoff[game,i] or UtopiaPayoff[i] represents the utopia payoff of player i.";

UtopiaSum::usage = 
"UtopiaSum[game,S_List] calculates the sum of utopia payoffs for coalition S.";

Gap::usage  = 
"Gap[game] balances the difference of the total utopia payoff \n
 of a coalition and its value."; 

AdjustedEffVector::usage = 
"AdjustedEffVector[game,i] calculates the adjusted efficient \n
 upper vector of player i for 1-convex games.";

AdjustedEffUpperVectors::usage = 
"AdjustedEffUpperVectors[game] calculates the adjusted efficient \n
 upper vector for all players for 1-convex games. There are exactly \n
 n upper efficient vectors in a 1-convex game."; 

AdjustedWorthVectors::usage = 
"AdjustedWorthVectors[game,k] computes the adjusted worth vectors \n 
 of k-convex games. k is a natural number.";
 
Concession::usage = 
"Concession[game] is the maximal concession of player i \n
 with respect to his utopia payoff UtopiaPayoff[game,i]"

DisagreeConvex::usage = 
"DisagreeConvex[game] calculates the disagreement vector for convex games."; 

Disagreement::usage = 
"Disagreement[game] calculates the disagreement vector of a game.";

TauValue::usage = 
"The TauValue[game] represents some efficient compromise \n
 between the upper bound 'Utopia-Payoff' and some lower bound \n
 'Disagreement' of the core.";  

MargValue::usage = 
"MargValue[game] calculates the marginal worth for all players.";

kCover::usage = 
"kCover[game,k] determines the k-cover of the game. k is a natural number.";

CoreElementsQ::usage = 
"CoreElementsQ[game,payoff] checks if a list of payoffs contain core allocations."; 

BelongToCoreQ::usage = 
"BelongToCore[game,payoff] checks if a list of payoffs contain core allocations. \n
 An imputation x is in the core iff x =< dual v."; 


ImputationQ::usage = 
"ImputationQ[game,payoff] checks whether the payoff or list of payoffs belongs to the imputation set.";

CoordS::usage = 
"CoordS[game,S] determines the unanimity coordinate of coalitions S.";


UnanimityCoordinates::usage = 
"UnanimityCoordinates[game] determines all unanimity coordinates of the game.";

MinUnanimityCoordinates::usage = 
"MinUnanimityCoordinates[game] calculates the minimum unanimity coordinate \n
 for each coalition size.";

StrictlyConvexUnanConditionQ::usage = 
"StrictlyConvexUnanConditionQ[game] checks at most (n-1) inequalities \n
 of the unanimity coordinates constraints for strict positive sums.  \n 
 It examines if the sufficient condition of convexity in terms of \n
 unanimity coordinates is satisfied. The return value is \n 
 'True' or 'False'. See also EvalSumMinCoord[].";     


ConvexUnanConditionQ::usage = 
"ConvexUnanConditionQ[game] checks at most (n-1) inequalities \n
 of the unanimity coordinates constraints for positive sums. \n
 It examines if the sufficient condition of convexity in terms of \n 
 unanimity coordinates is satisfied. The return value is \n 
 'True' or 'False'. See also EvalSumMinCoord[].";     


EvalSumMinCoord::usage = 
"EvalSumMinCoord[game] calculates at most (n-1) inequalities \n 
  of the unanimity coordinates constraints of nonnegative sums.\n
  It is a sufficient condition for convexity of the game.\n
  If all values are nonnegative then the game is convex. \n
  But the converse is not true."; 

WeightedMajority::usage = 
"WeightedMajority[T,weightList] determines a simple game from \n
 the nonnegative vector [q; p1, ..., pn], where T={1,...n}. \n
 It is admissible to specify the vector of weights by the following \n 
 lists {q, p1, ...,pn} or {q, {p1, ..., pn}} or {{q}, {p1, ...,pn}}. \n
 The simple game can then be defined by using the function DefineGame[]."


CallMaximize::usage = 
"Is an option for the function KernelCalculation[].
With this option the LP solver is changed from
ConstrainedMax/Min to Maximize/Minimize. Note
that the solver Maximize seems to be slower
than the default solver ConstrainedMax.
The solver ConstrainedMax will be deprecated in 
future Mathematica versions (>5)."

ChangeInternalEps::usage =
"Is an option for the function KernelCalculation[].
With this option it is possible to switch an
internal change of the epsilon value within the first loop 
off. This option is useful to obtain different 
allocations from the initial LPs."

DisplayAllResults::usage =
"Is an option for the functions Kernel[] and KernelCalculation[].
If the option is set to 'True', then all internal results are 
returned, otherwise the Kernel solution and the allocations
computed of the initial LPs are returned."

EpsilonValue::usage = 
"Is an option of the functions Kernel[] and KernelCalculation[].
The default value is set to zero. By using different real numbers,
the strong epsilon-core will be changed."

RationalApproximate::usage = 
"Is an option for the function DefineGame[].
The real numbers of the coalitional values
will be approximated by rational numbers. 
Default value is true."

RationalExact::usage = 
"Is an option for the functions CddGmpVerticesCore[] \n
and CddGmpPlotCore[]."


SetGameToNonZeroMonotonic::usage = 
"Is an option for the function KernelCalculation[]. To switch
this option on, it is possible to search for additional
pre-imputations for non-zero-monotonic games."

WithIncidences::usage = 
"Is an option for the function CddGmpVerticesCore[].
By switching the option on a facets-vertices incidence
matrix will be returned."


Which[$OperatingSystem === "Unix",
  Options[DefineGame] = {RationalApproximate -> True};
  Options[CddVerticesCore] = {RationalExact -> True};
  Options[CddGmpVerticesCore] = {RationalExact -> True, WithIncidences -> False};
  Options[CddGmpPlotCore] = {RationalExact -> True};
  Options[CddVerticesImputationSet] =  {RationalExact -> True};
  Options[CddVerticesReasonableSet] = {RationalExact -> True, ProperContribution -> False};
  Options[CddGmpPlotReasonableSet] = {ProperContribution -> False};
  Options[CddVerticesLowerSet] = {RationalExact -> True};
  Options[CddGmpPlotLowerSet] = {};
  Options[Kernel] = {CallMaximize -> False, DisplayAllResults -> False, EpsilonValue -> 0};
  Options[KernelCalculation] = {CallMaximize -> False, ChangeInternalEps -> False, DisplayAllResults -> False, EpsilonValue -> 0, SetGameToNonZeroMonotonic -> False};
  Options[KernelPoints] = {EpsilonValue -> 0, SetGameToNonZeroMonotonic -> False};
  Options[KernelSubCallEmpty] = {EpsilonValue -> 0, SetGameToNonZeroMonotonic -> False};
  Options[KernelSubCall] = {EpsilonValue -> 0, SetGameToNonZeroMonotonic -> False};
  Options[KernelSubCallZero] = {EpsilonValue -> 0, SetGameToNonZeroMonotonic -> False};
  Options[ModifiedKernel] = {CallMinimize -> True, KernelRange -> False, SetRecursionLimit -> 128, Silent -> True};
  Options[KernelVertices] = {CallMaximize -> True, EpsilonValue -> 0, Silent -> True};
  Options[ModifiedNucleolus] = {CallMinimize -> True, SetRecursionLimit -> 128, Silent -> True};
  Options[LexiCenter] = {CallMinimize -> True, SetRecursionLimit -> 128, Silent -> True};
  Options[PreNucleolus] = {CallMinimize -> True, SetRecursionLimit -> 128, Silent -> True};
  Options[AllMaxSurpluses] = {DisplayMatrixForm -> False};
  Options[ExcessPayoff] = {DisplayMatrixForm -> False};
  Options[FindKernelSolution] = {DigitPrecision -> 6, RationalTol -> 10^(-7), SetRecursionLimit -> 512, Silent -> True};
  Options[FindPreKernelSolution] = {DigitPrecision -> 6, RationalTol -> 10^(-7), SetRecursionLimit -> 512, Silent -> True}; 
  Options[UpperSetIncImputationQ] = {Silent -> True};
  Options[LowerSetIncImputationQ] = {Silent -> True};
  Options[PreKernelEqualsKernelQ] = {Silent -> True};,
True,
  Options[DefineGame] = {RationalApproximate -> True};
  Options[CddVerticesCore] = {RationalExact -> False};
  Options[CddGmpVerticesCore] = {RationalExact -> True, WithIncidences -> False};
  Options[CddGmpPlotCore] = {RationalExact -> True};
  Options[CddVerticesImputationSet] =  {RationalExact -> False};
  Options[CddVerticesReasonableSet] = {RationalExact -> False, ProperContribution -> False};
  Options[CddGmpPlotReasonableSet] = {ProperContribution -> False};
  Options[CddGmpPlotLowerSet] = {};
  Options[CddVerticesLowerSet] = {RationalExact -> False};
  Options[Kernel] = {CallMaximize -> False, DisplayAllResults -> False, EpsilonValue -> 0};
  Options[KernelCalculation] = {CallMaximize -> False, ChangeInternalEps -> False, DisplayAllResults -> False, EpsilonValue -> 0, SetGameToNonZeroMonotonic -> False};
  Options[KernelPoints] = {EpsilonValue -> 0, SetGameToNonZeroMonotonic -> False};
  Options[KernelSubCallEmpty] = {EpsilonValue -> 0, SetGameToNonZeroMonotonic -> False};
  Options[KernelSubCall] = {EpsilonValue -> 0, SetGameToNonZeroMonotonic -> False};
  Options[KernelSubCallZero] = {EpsilonValue -> 0, SetGameToNonZeroMonotonic -> False};
  Options[ModifiedKernel] = {CallMinimize -> True, KernelRange -> False, SetRecursionLimit -> 128, Silent -> True};
  Options[KernelVertices] = {CallMaximize -> True, EpsilonValue -> 0, Silent -> True};
  Options[ModifiedNucleolus] = {CallMinimize -> True, SetRecursionLimit -> 128, Silent -> True};
  Options[LexiCenter] = {CallMinimize -> True, SetRecursionLimit -> 128, Silent -> True};
  Options[PreNucleolus] = {CallMinimize -> True, SetRecursionLimit -> 128, Silent -> True};
  Options[AllMaxSurpluses] = {DisplayMatrixForm -> False};
  Options[ExcessPayoff] = {DisplayMatrixForm -> False};
  Options[FindKernelSolution] = {DigitPrecision -> 6, RationalTol -> 10^(-7), SetRecursionLimit -> 512, Silent -> True};
  Options[FindPreKernelSolution] = {DigitPrecision -> 6, RationalTol -> 10^(-7), SetRecursionLimit -> 512, Silent -> True}; 
  Options[UpperSetIncImputationQ] = {Silent -> True};
  Options[LowerSetIncImputationQ] = {Silent -> True};
  Options[PreKernelEqualsKernelQ] = {Silent -> True};
]

Begin["`Private`"]

(* Check the directory where you have installed your packages *)

Needs["coop`CooperativeGames`"]
Needs["TuGamesAux`"]
(* Needs["Algebra`InequalitySolve`"]*)
Which[$OperatingSystem === "Unix", Needs["VertexEnum`"],
      $OperatingSystem === "Windows", Needs["vertex`VertexEnum`"],
      True, Needs["VertexEnum`"]
]
      

(* Comments for Windows OS starts
   Check the paths were you have installed the cddmathlink library.
   The default value is set to:  
   SetDirectory["C:\TEMP\cddlm"]
   If you have installed the mathlink library in the Mathematica subdirectory, then
   you should use no space in the directory names to avoid error messages and to be sure
   that the library can be found. For instances, set in this case 
   SetDirectory["C:\Programme\WolframResearch\Mathematica\4.2\AddOns\Applications\TuGames\cddlm"]
   cddml=Install["cddml_w32new"] 
   Comments for Windows OS ends 
*)

Which[$OperatingSystem === "Unix",
        Which[$MachineName === "xxx", SetDirectory["/usr/local/bin"],
        $MachineName === "xxx", SetDirectory["/usr/local/bin"],
        $MachineName === "xxx-e202", SetDirectory["/home/local/ws/xxx/bin64"],
        $MachineName === "xxx-e101", SetDirectory["/home/local/ws/xxx/bin"],
        $MachineName === "xxx-e102", SetDirectory["/home/local/ws/xxx/bin"],
        $MachineName === "xxx-e103", SetDirectory["/home/local/ws/xxx/bin"],
        $MachineName === "xxx-e000", SetDirectory["/home/local/ws/xxx/bin"],
        $MachineName === "xxx-e001", SetDirectory["/home/local/ws/xxx/bin"],
        $MachineName === "xxx-e002", SetDirectory["/home/local/ws/xxx/bin"],
        $MachineName === "xxx-e003", SetDirectory["/home/local/ws/xxx/bin"],
        $MachineName === "xxx", SetDirectory["/home/dfs/lh00/xxx/bin"],
        $MachineName === "xxx", SetDirectory["/home/dfs/lh00/xxx/bin"],
        True, SetDirectory["/usr/local/bin"]
       ],
      $OperatingSystem === "Windows", 
         Which[$VersionNumber === 4.2, SetDirectory["C:\\ProgrammeWolframResearch\\Mathematica\\Ver42\\AddOns\\Applications\\TuGames\\cddlm"],
               $VersionNumber === 5., SetDirectory["C:\\Programme\\WolframResearch\\Mathematica\\Ver50\\AddOns\\Applications\\TuGames\\cddlm"],
               True, SetDirectory["C:\\TEMP\\cddlm"]
              ]
]

(*  Deactivate the lines below if you have not installed cddmathlink. *)

Which[$OperatingSystem === "Unix",
         cddml=Install["./cddmathlink"];
         cddmlgmp = Install["./cddmathlink2gmp"];
         cddml2=Install["./cddmathlink2"];, 
(* cddmlgmp=Install["./cddmathlink2"] *)
      $OperatingSystem === "Windows", 
         cddml=Install["cddml_w32new"]; 
]

Off[ConstrainedMin::deprec]
Off[ConstrainedMax::deprec]



(* Section general functions starts *)


DefineGame[R_,values_,opts___?OptionQ]:=(Clear[T];T=R;Clear[v];
    approxrat =  RationalApproximate /. {opts} /. Options[DefineGame];    
    If[(approxrat && $VersionNumber >= 5.) == True, MapThread[Set,{Map[v,Coalitions],Rationalize[values, 5^(-12)]}], MapThread[Set,{Map[v,Coalitions],values}]])

(*
DefineGame[R_,values_]:=(Clear[T];T=R;Clear[v];
    MapThread[Set,{Map[v,Coalitions],values}])
*)

DualGame[game_]:=(v[T]-v[Complement[#]]& /@ Reverse[Coalitions])

(*
Def.: Let S_1,S_2,...S_k be a partition of the grand coalition T, then
the game v is called weakly superadditive, if sum_{i=1}^k v(S_i) <= v(T). 
	
Prop.:
v is a weakly superadditive game if and only if v <= dual v
*)


WeaklySuperAdditiveQ[game_] := Module[{truelist, wahrQ},
    truelist = Thread[(v[#] & /@ Coalitions) <= DualGame[game]];
    wahrQ = Apply[And, truelist];
    wahrQ
]



(*
Def. Every subset S of T induce a subgame w via w(S) = v(S) on the new player set S.
Prop. A game v is superadditive, if for every subgame w it holds w =< dual w.
*)

SuperAdditiveQ[game_] := Module[{coal, retval},
(* 
Switching off warnings that Mathematica tries to assign values to protected variables.
*)
    Off[Clear::wrsym]; 
    Off[Set::setraw];
    Off[Set::write];
    coal = Coalitions;
    SetAttributes[v, Protected];
    retval = SuperCheck[game, coal];
    ClearProtected[v];
(* Switching on the warnings *)
    On[Clear::wrsym]; 
    On[Set::setraw];
    On[Set::write];
    retval
    ]

ClearProtected[Global`w_] := (Unprotect[Global`w]; Clear[Global`w])

(*
Checking if w =< dual w holds for every subgame w.
*)

SuperCheck[game_, liste_] := 
  Module[{mengen = liste, obermenge, subgame, dualval, gensub, newsets, val, 
      truelist = {}, wahrQ},
    While[Length[mengen] > 0,
      obermenge = Take[mengen, -1][[1]];
      MapThread[Set, {{Global`w[#]}, {v[#]}}] & /@ Subsets[obermenge];
      SetAttributes[Global`w, Protected];
      newsets = DeleteCases[mengen, obermenge];
      gensub = Subsets[obermenge];
      val = Global`w[#] & /@ gensub;
      subgame = DefineGame[obermenge, val];
      subdual = DualGame[subgame];
      pruef = MapThread[GreaterEqual, {subdual, val}];
      mengen = newsets;
      ClearProtected[Global`w];
      wahrQ = Apply[And, pruef];
      truelist = Append[truelist, wahrQ];
     ];
    Apply[And, truelist]
    ]

SelectSuperSets[i_]:=W[i]

W[i_]:= Cases[Coalitions,{___,i,___}]

Assign[values_]:=Thread[Rule[(v[#]& /@ Coalitions),values]]

WeightedMajority[T_,weight_List] := Module[{outersets,coalweights,sumweights,threshold,boolval,un,trq,pos},
       Off[Part::partd];
       outersets = Outer[List, #] & /@ Subsets[T];
       coalweights = Which[Depth[weight] === 3, Extract[Flatten[Drop[weight,1]], #] & /@ outersets, Depth[weight] === 2,Extract[Drop[weight,1], #] & /@ outersets];
       sumweights = Apply[Plus, #] & /@ coalweights;
       threshold = Which[Depth[weight] == 3, un = Union[Flatten[{First[weight][[1]],First[weight]}]];
                                               Which[Length[un] === 0, un, 
                                                     True, trq = NumberQ[#] &/@ un;
                                                     pos = Position[trq,True];
                                                     Extract[un,Flatten[pos]]
                                               ], 
                         Depth[weight] == 2, First[weight]];
       On[Part::partd];
       boolval = GreaterEqual[#, threshold] & /@ sumweights;
       Which[# === True, 1, True, 0] & /@ boolval
]


(* Section general functions ends *)


(* convexity and av-convexity section starts *)


IncreasMargContributions[game_,i_ ,values_]:=
	Do[Print[(v[#]-v[DeleteCases[#,i]] <= 
         v[Union[Flatten[{#,j}]]] - v[DeleteCases[ 
          Union[Flatten[{#,j}]],i]])& /@ Take[W[i],Length[W[i]]-1]/.Assign[values]],
           {j,1,Length[T]}]

IncreasingMargContributions[game_,i_ ]:=
	Table[(v[#]-v[DeleteCases[#,i]] <= 
          v[Union[Flatten[{#,j}]]] - v[DeleteCases[ 
            Union[Flatten[{#,j}]],i]])& /@ Take[W[i],Length[W[i]]-1] ,
              {j,1,Length[T]}]
			
ConvCheck[game_]:= Table[IncreasingMargContributions[game,i],{i,Length[T]}]

ConvexQ[game_]:= Module[{liste}, 
                  liste = Flatten[Table[IncreasingMargContributions[game,i],{i,Length[T]}],1];
		  Apply[And,Apply[And,liste,1]]
	]


StrIncreasMargContrib[game_,i_ ]:=
	Table[(Rationalize[v[#] - v[DeleteCases[#,i]]] < 
          Rationalize[v[Union[Flatten[{#,j}]]] - v[DeleteCases[ 
             Union[Flatten[{#,j}]],i]]])& /@ Take[W[i],Length[W[i]]-1] ,
              {j,1,Length[T]}]
			

ConvStrQ[game_]:= Table[StrIncreasMargContrib[game,i],{i,Length[T]}]

ConvexStrQ[game_]:= Module[{liste},
		liste = Flatten[Table[StrIncreasMargContrib[game,i],{i,Length[T]}],1];
		 Apply[And,Apply[And,liste,1]]
	]
	
	
AvConvexQ[game_] := Module[{trueQ},
    trueQ = Table[AvLoop[game, W[i]], {i, 1, Length[T]}];
    Apply[And, trueQ]
    ]
	
AvLoop[game_, liste_] := Module[{el = liste, R, wa = {}}, 
       While[Length[el] > 0,
        R = Take[el, -1][[1]];
        erzli = Table[Blank[], {i, 1, Length[R]}];
        li = DeleteCases[el, erzli];
        tr = AvSumCheck[game, R, li];
        wa = Append[wa, tr];
        wahrQ = Apply[And, Flatten[wa]];
        el = Drop[el, -1]];
      wahrQ
      ];

	
ContributionSum[R_, S_] := Module[{va1},
    va1 = 
      Apply[Plus, 
        Table[v[R] - v[DeleteCases[R, Cases[R, R[[j]]][[1]]]], {j, 1, 
            Length[S]}]]]
            
ContributionSum[game_, R_, S_] := Module[{va1},
    va1 = 
      Apply[Plus, 
        Table[v[R] - v[DeleteCases[R, Cases[S, S[[j]]][[1]]]], {j, 1, 
            Length[S]}]]]
            
AvSumCheck[game_,R_,liste_]:= Module[{li, po, newliste},
        li = Table[R == Union[R, liste[[i]]], {i, 1, Length[liste]}];
        po = Position[li, False];
        newliste = Delete[liste, po];
        MapThread[(ContributionSum[game, #1, #2] <= ContributionSum[game, R, \
#2]) &, {newliste, 
        newliste}]]       


(* convexity and av-convexity section ends *)
      

(* (zero)-monotonicity section starts *)

ZeroMonotoneQ[game_] := Module[{zeroval, newgame},
    zeroval = ZeroNormalization[game];
    newgame = DefineGame[T, zeroval];
    MonotoneQ[newgame]
    ]

ZeroNormalization[game_] := Module[{genIndVal, addIndVal, coalval},
    coalval = v[#] & /@ Coalitions ;
    genIndVal = Map[v, #] & /@ Outer[List, Coalitions];
    addIndVal = Apply[Plus, #] & /@ genIndVal;
    Subtract[coalval, addIndVal]
    
    ]


ZeroOneNormalization[game_] := Module[{zeroli, div},
    zeroli = ZeroNormalization[game];
    div = Divide[ #, Take[zeroli, -1]] & /@ zeroli;
    Flatten[div]
    ]


OneNormalization[game_] := Module[{newgame},
    newgame = v[#] & /@ Subsets[T];
    newgame * (1/Last[newgame])

]

(*
A game v is monotone if v(S \cup i) - v(S) is non-negative for all i \in N.
*)


GameMonotoneQ[game_]:= Module[{liste}, 
                  liste = Flatten[Table[MargContribPositive[game,i],{i,Length[T]}],1];
		  Apply[And,Apply[And,liste,1]]
	]



MargContribPositive[game_,i_ ]:= Module[{margli,grQ},
        margli = ( v[#]-v[DeleteCases[#,i]])& /@ W[i];
        grQ = MapThread[GreaterEqual,{margli,Table[0,{j,Length[margli]}]}];
        grQ
 ]




MonotoneQ[game_] := Module[{coal},
    coal = Coalitions;
    MonoCheck[game, coal]
    ]


(* Check monotonicity for every subgame  *)

MonoCheck[game_, liste_] := 
  Module[{mengen = liste, obermenge, gensub, newsets, val, truelist = {}, 
      wahrQ},
    While[Length[mengen] > 0,
      obermenge = Take[mengen, -1][[1]];
      newsets = DeleteCases[mengen, obermenge];
      gensub = Subsets[obermenge];
      val = v[#] & /@ gensub;
      pruef = (val[[Length[gensub]]] >=  #) & /@ val;
      mengen = newsets;
      wahrQ = Apply[And, pruef];
      truelist = Append[truelist, wahrQ];
     ];
    Apply[And, truelist]
    ]

(* (zero)-monotonicity section ends *)




(* strong epsilon core section starts *)

EpsValue[game_, eps_] := Map[Set[v[#], v[#] - eps] &, ProperCoalitions]
      
     
NewRelax[ineq_, epsilon_] := ineq /. lhs_ >= rhs_ -> lhs >= rhs - epsilon
      
EpsCore[game_] := ConstrainedMin[Global`e - f, 
        Append[NewRelax[CoalitionalRationality,Global`e - f], \
ParetoOptimality], 
        Join[PayoffVector, {Global`e, f}]]

(* Section critical values starts *)
(* The following critical values are due to Maschler, Peleg and Shapley (1979) *)
            
FirstCriticalVal[game_] := Module[{con, critical},
          con = ConstrainedMin[e - f, Append[
          NewRelax[CoalitionalRationality, e - f],
          ParetoOptimality], Join[PayoffVector, {e, f}]];
          critical = MapThread[Rule, {{Global`eps1}, {con[[1]]}}]]
        
SecondCriticalVal[game_] := Module[{cval2},
         cval2 = Max[(v[#] - Apply[Plus, Map[v[{#1}] &, #]]) & /@ ProperCoalitions];
         MapThread[Rule, {{Global`eps2}, {cval2}}]
    ]
            
 
ThirdCriticalVal[game_] := Module[{cval3}, 
    ReasonableOutcome[game];
    cval3 = Max[(v[#] - v[T] + Apply[Plus, Map[Global`r[#1] &, Complement[T,#]]]) & /@ ProperCoalitions];
         MapThread[Rule, {{Global`eps3}, {cval3}}]
     ]
    
FourthCriticalVal[game_]:= Module[{cval}, 
     cval4 = Min[{SecondCriticalVal[game][[1,2]],ThirdCriticalVal[game][[1,2]]}];
                MapThread[Rule, {{Global`eps4}, {cval4}}]
        ]
    
StarCriticalVal[game_] :=Module[{cval, upbd},
        upbd = MapThread[Rule, {up[#] & /@ T, ReasonableOutcome[game]}];
        cval =  Max[Min[{v[#] - Apply[Plus, v[{#1}] & /@ #], 
            v[#] + (up[Complement[T, #]] /. upbd) - v[T]}] & /@ 
         ProperCoalitions]; 
      MapThread[Rule, {{Global`stareps}, {cval}}]
   ]

up[S_List] := Plus @@ up /@ S
m[S_List] := Plus @@ m /@ S

 
(* The next two critical values are defined by Chang and Kan (1992) and Chang and Driessen (1994) *)


SecondStarCriticalVal[game_] := Module[{lwbd, lwpara, upbd, uppara, dstarcval},
    lwbd = SmallestContributionVector[game];
    lwpara = MapThread[Rule, {m[#] & /@ T, lwbd}];
    upbd = ReasonableOutcome[game];
    uppara = MapThread[Rule, {up[#] & /@ T, upbd}];
    dstarcval = Which[Max[v[#] & /@ Drop[Subsets[T], -1]] >  v[T] === False,
                        Which[LowerSetQ[game] === False, CaseOne[game, uppara],
                         True, Which[CoreQ[game] === False, CaseTwo[game, lwpara, uppara],
                                        True, CaseThree[game, lwpara, uppara]]
             ],
        True, StarCriticalVal[game][[1, 2]]
        ];
    MapThread[Rule, {{Global`dstareps}, {dstarcval}}]
    ]

ThirdStarCriticalVal[game_] := Module[{lwbd, lwpara, upbd, uppara, thstarcval},
    lwbd = SmallestContributionVector[game];
    lwpara = MapThread[Rule, {m[#] & /@ T, lwbd}];
    upbd = ProperAmount[game];
    uppara = MapThread[Rule, {up[#] & /@ T, upbd}];
    thstarcval = Which[Max[v[#] & /@ Drop[Subsets[T], -1]] >  v[T] === False,
                        Which[((LowerSetQ[game] || UpperSetQ[game]) && CoreQ[game]) === True, 0,
                         True, Which[CoreQ[game] === False, CaseTwo[game, lwpara, uppara],
                                        True, CaseThreeFor3[game, lwpara, uppara]]
             ],
        True, StarCriticalVal[game][[1, 2]]
        ];
    MapThread[Rule, {{Global`thstareps}, {thstarcval}}]
    ]

CaseOne[game_, uppara_List] := Module[{},Max[Min[{(v[#] + up[Complement[T, #]] - v[T]), v[#] - Apply[Plus, v[{#1}] & /@ #], 0}] & /@ ProperCoalitions  /. uppara]];
CaseTwo[game_, lwpara_List, uppara_List] := Module[{}, Max[Min[{v[#] - m[#], v[#] - Apply[Plus, v[{#1}] & /@ #], v[#] + up[Complement[T, #]] - v[T]}] & /@ ProperCoalitions  /. lwpara  /. uppara]];
CaseThree[game_, lwpara_List, uppara_List] := Module[{},Max[Min[{Max[{v[#] - m[#], 0}], v[#] - Apply[Plus, v[{#1}] & /@ #], (v[#] + up[Complement[T, #]]  - v[T])}] & /@ ProperCoalitions  /. lwpara /. uppara]];
CaseThreeFor3[game_, lwpara_List, uppara_List] := Module[{},Max[Min[{Max[{Min[{v[#] - Apply[Plus, v[{#1}] & /@ #],v[#] - m[#]}],0}], Max[{(v[#] + up[Complement[T, #]] - v[T]),0}]}] & /@ ProperCoalitions  /. lwpara  /. uppara]];


FifthCriticalVal[game_] := Module[{lwbd, lwpara,cval5},
    lwbd = SmallestContributionVector[game];
    lwpara = MapThread[Rule, {m[#] & /@ T, lwbd}];
    cval5=Which[LowerSetQ[game] === False, 0,
                 LowerSetQ[game] === True,
      Which[ CoreQ[game] === False, Max[(v[#] - (m[#] /. lwpara)) & /@ ProperCoalitions],
       True, Max[Max[{(v[#] - (m[#] /. lwpara)), 0}] & /@ ProperCoalitions]] 
      ];
    MapThread[Rule, {{Global`eps5}, {cval5}}]
   ]

UpperSetQ[game_] := Module[{upbd,uppara},
    upbd = ProperAmount[game];
    uppara = MapThread[Rule, {up[#] & /@ T, upbd}];
    (up[T] /. uppara) >=  v[T]
    ]


LowerSetQ[game_] := Module[{lwbd, lwpara},
    lwbd = SmallestContributionVector[game];
    lwpara = MapThread[Rule, {m[#] & /@ T, lwbd}];
    (m[T] /. lwpara) <=  v[T]
    ]

LowerSetIncImputationQ[game_,opts___?OptionQ] := Module[{sil},
   sil = Silent /. {opts} /. Options[LowerSetIncImputationQ];
   Which[sil===False,DisplayMessageLower;True];
   Min[SmallestContributionVector[game]] >= 0
] 

DisplayMessageLower:=(Print["Checking if Kernel and Pre-kernel are identical"];
                      Print["True is equivalent to zero-monotonicity of the game."];)

UpperSetIncImputationQ[game_,opts___?OptionQ] := Module[{sil,prub, prpara, setwoi},
    sil = Silent /. {opts} /. Options[UpperSetIncImputationQ];
    Which[UpperSetQ[game] === True,
      prub = ProperAmount[game];
      prpara = MapThread[Rule, {up[#] & /@ T, prub}];
      setwoi = DeleteCases[T, #] & /@ T;
      Which[sil===False,DisplayMessageUpper;True];
      v[T] >= Max[up[#] & /@ setwoi /. prpara],
      True, Print["Upper Set is empty. Nothing to check."]]]


DisplayMessageUpper:=(Print["This class of games is different from zero-monotonic games."];
                      Print["If True, then the pre-kernel is equal to the kernel."];);

IntersectionUpperLowerSetQ[game_] := Module[{},
     LowerSetQ[game] && UpperSetQ[game]
   ] 

GrandCoalitionLargestValueQ[game_] := Module[{},
  Max[v[#] &/@ Subsets[T]] <= v[T] 
  ]

PreKernelEqualsKernelQ[game_,opts___?OptionQ] := Module[{sil},
  sil = Silent /. {opts} /. Options[PreKernelEqualsKernelQ];
  Which[sil===False,
         Switch[True, ZeroMonotoneQ[game],Print["Checking if the game is zero monotone."]; True,
                      (!(!LowerSetQ[game]) && (!UpperSetQ[game])), Print["Checking if the intersection of the upper and lower set is empty."]; True,
                       UpperSetIncImputationQ[game], Print["Checking if the upper set is included in the imputation set."];True,
(*  This statement needs more!                     (GrandCoalitionLargestValueQ[game] && IntersectionUpperLowerSetQ[game]),
                                        Print["Checking if the intersection of the upper and lower set is non empty"];
                                        Print["and that the grand coalition has the largest value."];
                                        True,*)
                True, Print["Checked sufficient conditions are not satisfied."];False
         ],
        True,
          Switch[True, ZeroMonotoneQ[game], True,
                     (!(!LowerSetQ[game]) && (!UpperSetQ[game])), True,
                     UpperSetIncImputationQ[game], True,
(*                     (GrandCoalitionLargestValueQ[game] && IntersectionUpperLowerSetQ[game]),True,*)
                      True,False
      ]
    ]
  ]


(* Section critical values ends *)

LargestAmount[game_, i_] := Max[(v[#] - v[DeleteCases[#, i]]) & /@ W[i]]
LargestAmount[game_] := LargestAmount[game,#] &/@ T
 
ProperAmount[game_, i_] := Max[(v[#] - v[DeleteCases[#, i]]) & /@ Drop[W[i], -1]]
ProperAmount[game_] := ProperAmount[game,#] &/@ T

ImplementReas[game_] := Map[Global`r, T]
 
ReasonableOutcome[game_] := (Clear[Global`r]; MapThread[Set, {ImplementReas[game], \
LargestAmount[game]}]);
 
ReasonableConst := x[#] <= Global`r[#] & /@ T;
 
ReasonableSet[game_] := Module[{assr},
     ReasonableOutcome[game];
    assr = (Clear[Global`r]; MapThread[Set, {ImplementReas[game], LargestAmount[game]}]);
    Append[ReasonableConst, ParetoOptimality]
    ]
 
(* strong epsilon core section ends *) 


NewShapley[game_]:= Module[{margvec},
   margvec = MargValue[game];
   Apply[Plus,margvec]/Factorial[Length[T]]
]


PrintRemark[payoff_]:= (
      Print["Depth is equal to ",Depth[payoff]];
      Print["Usage: MaxExcessBalanced[game,payoff] and ExcessPayoff[game,payoff]"];
      Print["The variable 'payoff' has not the correct input format."];
      Print["Input format of the variable 'payoff' is not correct."];
      Print["The variable 'payoff' must be a list of payoff vectors or a single payoff vector."]
);
 
ExcessPayoff[game_,payoff_,opts___?OptionQ]:= Module[{dispmat,assg,li,res},
  dispmat = DisplayMatrixForm /. {opts} /. Options[ExcessPayoff];    
  If[Depth[payoff] == 2 || Depth[payoff] == 3,
    li = If[Length[Dimensions[payoff]]==1,{payoff},payoff];
    assg = Table[MapThread[Rule,{Map[x,T],li[[i]]}],{i,Length[li]}];
    res  = Table[((v[#]-x[#])& /@ Coalitions)/.assg[[i]],{i,Length[assg]}];
    Which[dispmat == False, res, True, DisplayErgb[res]],
            PrintRemark[payoff]
		]
     ]

DisplayErgb[payoff_]:= Module[{exc,coal,mpc},
    exc = payoff;
    coal = Subsets[T];
    mpc = Map[Global`Co,coal];
    MatrixForm[PrependTo[exc,mpc]]
]

MaxExcessSets[game_, payoff_] := Module[{exclist, propexc, maxexc, pos},
    exclist = ExcessPayoff[game, payoff];
    propexc = Drop[Drop[#, 1], -1] & /@ exclist;
    maxexc = Max[#] & /@ propexc;
    pos = MapThread[Position[#1, #2] &, {propexc, maxexc}];
    Extract[ProperCoalitions, #] & /@ pos
    ]

IntersectionOfMaxExcessSets[game_, payoff_List] := Module[{dsets},
   dsets = MaxExcessSets[game, payoff];
   Which[GrandCoalitionLargestValueQ[game]===False,DisplayMessFalse[game,payoff],
         True, DisplayMessTrue[game,payoff]];
   Apply[Intersection, #] & /@ dsets
  ]

DisplayMessFalse[game_,payoff_List] := (Print["Grand Coalition has not largest value."];
                            Print["Payoff vector/s is/are kernel element(s)? ", KernelImputationListQ[game,payoff]];)
DisplayMessTrue[game_,payoff_List] := (Print["If kernel payoff, then the intersection set is empty."];
                           Print["Payoff vector/s is/are kernel element(s)? ", KernelImputationListQ[game,payoff]];)


TIJsets[i_,j_]:=DeleteCases[Cases[ProperCoalitions,{___,i,___}],{___,j,___}];

MaxExcessBalanced[game_,payoff_]:= Module[{assg,li,sij,rsij,bal1,bal2},
  If[Depth[payoff] == 2 || Depth[payoff] == 3,
(*     TIJsets[i_,j_]:=DeleteCases[Cases[ProperCoalitions,{___,i,___}],{___,j,___}]; *)
     sij = Flatten[Table[Table[ \
       TIJsets[i,j],{j,i+1,Length[T]}],{i,Length[T]}],1];
     sjiset = \
          Flatten[Table[Table[TIJsets[j,i],{j,i+1,Length[T]}],{i,Length[T]}],1];
     li = If[Length[Dimensions[payoff]]==1,{payoff},payoff];
     assg = Table[MapThread[Rule,{Map[x,T],li[[i]]}],{i,Length[li]}];
     bal1=Rationalize[Table[
        Table[Max[((v[#]-x[#])& /@ sij[[i]])/.assg[[k]]],{i,Length[sij]}],{k,
          Length[assg]}]];
     bal2=Rationalize[Table[
        Table[Max[((v[#]-x[#])& /@ sjiset[[i]])/.assg[[k]]],{i,
            Length[sjiset]}],{k,Length[assg]}]];
		Table[Apply[And,MapThread[Equal,{bal1[[i]],bal2[[i]]}]],{i,Length[bal1]}],
              PrintRemark[payoff]
		]
    ]



Quota[game_]:= Module[{secsize,matrix},
		 secsize = Reverse[Table[v[DeleteCases[T,i]],{i, Length[T]}]];
		 matrix = Reverse[Table[If[i==j,0,1],{i,Length[T]},{j,Length[T]}]];   
                 LinearSolve[matrix,secsize]
		   ]

VerticesCore[game_]:=
		Module[{anf,liste,mat,values},
		anf =T; 
		liste = Table[If[i>=j,1,0],{i,Length[T]},{j,Length[T]}];
		mat = Flatten[Table[Permutations[liste[[i]]],{i,Length[liste]}],1];
		values =v[#]& /@ Rest[Coalitions] ;
		extrem= VertexEnumeration[-mat,-values]
	  ]


(* Nucleolus/Kernel section starts *)
(* See M. Maschler 1992 for a detailed description of the algorithm (Kopelowitz Algorithm). *)



ModifiedNucleolus[game_, opts___?OptionQ] :=
  Module[{reclim, propsets, callm, sil, ineq, res, nucval,orgval,zeroval,newgame},
    reclim = SetRecursionLimit /. {opts} /. Options[ModifiedKernel];
    propsets = ProperCoalitions;
    callm = CallMinimize /. {opts} /. Options[ModifiedKernel];
    sil = Silent /. {opts} /. Options[ModifiedKernel];
    ineq = False;
    nucval = True;
    orgval = Rationalize[v[#] & /@ Subsets[T]];
    zeroval = ZeroOneNormalization[game];
    newgame = DefineGame[T,zeroval];
    res = Block[{$RecursionLimit = reclim},CallLP[newgame, propsets, callm, sil, ineq, nucval]];
    DefineGame[T,orgval]; (* Reconstruction of the original game. *)
    fires=ReMapResult[game,res,zeroval];
    Return[fires]
    ]

ModifiedKernel[game_, opts___?OptionQ] :=
  Module[{reclim, propsets, callm, sil, ineq, res, nucval,fires},
    reclim = SetRecursionLimit /. {opts} /. Options[ModifiedKernel];
    propsets = ProperCoalitions;
    callm = CallMinimize /. {opts} /. Options[ModifiedKernel];
    sil = Silent /. {opts} /. Options[ModifiedKernel];
    ineq = KernelRange /. {opts} /. Options[ModifiedKernel];
    nucval = False;
    orgval = Rationalize[v[#] & /@ Subsets[T]];
    zeroval = ZeroOneNormalization[game];
    newgame = (DefineGame[T,zeroval]);
    res = Block[{$RecursionLimit = reclim},CallLP[newgame, propsets, callm, sil, ineq, nucval]];
    DefineGame[T,orgval]; (* Reconstruction of the original game. *)
    fires=ReMapResult[game,res,zeroval];
    Which[Depth[res]>2,PrintWarning;Return[{fires,res[[2]]}],True,fires]
    ]


PreNucleolus[game_, opts___?OptionQ] :=
  Module[{reclim, propsets, callm, sil, ineq, res,fires,lexival},
    reclim = SetRecursionLimit /. {opts} /. Options[PreNucleolus];
    propsets = ProperCoalitions;
    callm = CallMinimize /. {opts} /. Options[PreNucleolus];
    sil = Silent /. {opts} /. Options[PreNucleolus];
    ineq = False;
    lexival=False;
    orgval = Rationalize[v[#] & /@ Subsets[T]];
    zeroval = ZeroOneNormalization[game];
    newgame = (DefineGame[T,zeroval]);
    res = Block[{$RecursionLimit = reclim},OldcallLP[newgame, propsets, callm, sil, ineq,lexival]];
    DefineGame[T,orgval]; (* Reconstruction of the original game. *)
    fires=ReMapResult[game,res,zeroval];
    Return[fires]
    ]

LexiCenter[game_, opts___?OptionQ] :=
  Module[{reclim, propsets, callm, sil, ineq, res,fires,lexival},
    reclim = SetRecursionLimit /. {opts} /. Options[LexiCenter];
    propsets = ProperCoalitions;
    callm = CallMinimize /. {opts} /. Options[LexiCenter];
    sil = Silent /. {opts} /. Options[LexiCenter];
    ineq = False;
    lexival=True;
    orgval = Rationalize[v[#] & /@ Subsets[T]];
    zeroval = ZeroOneNormalization[game];
    newgame = (DefineGame[T,zeroval]);
    res = Block[{$RecursionLimit = reclim},OldcallLP[newgame, propsets, callm, sil, ineq, lexival]];
    DefineGame[T,orgval]; (* Reconstruction of the original game. *)
    fires=ReMapResult[game,res,zeroval];
    Return[fires]
    ]

PrintWarning:=(Print["Warning: Internal computation was based on the Zero-One Normalization."];
               Print["Kernel Range solution must be re-mapped to the original game."]);

ReMapResult[game_,result_List,valuezero_List]:=Module[{indrat,sumirat,indgrz,grval,zeroess},
   indrat= v[{#}] &/@T;  
   sumirat = Apply[Plus,indrat];
   grval = Take[valuezero,-1];
   indgrz = Max[indrat]>0;
   zeroess = Apply[Plus,indrat]<=grval[[1]];
   Which[(zeroess && indgrz) === True, Which[Depth[result] >2,result[[1]]+indrat,True,result+indrat],
         (zeroess && (!indgrz)) === True, Which[Depth[result] >2,result[[1]]*v[T],True,result*v[T]],
         ((!zeroess) && indgrz) ===True, Which[Depth[result] >2, indrat + (v[T]-sumirat)*result[[1]],True,indrat + (v[T]-sumirat)*result],
         True, PrintNoRule;result
  ]
]

PrintNoRule:=(Print["No rule found to remap the result from a zero-monotonic game back to the original game."];);


CallLP[game_, mengen_List, consets_:{}, storeres_:{},callm_, sil_, ineq_, nucval_,counter_:1] := 
  Module[{gzlvQ,upb,uplwc,const01,const02,const,var,erg,alpres,intres,oldres,oldalp,alterg,exc,pos,uniqsets,bnewsets,balconst,zf,solcoef,t1,balcoef,zerocfQ,balpos,balsets,nonbal,nsets,const03,solcon,const04},
    gzlvQ = ((!ZeroMonotoneQ[game]) && (!GrandCoalitionLargestValueQ[game]));
    upb = Which[gzlvQ === False, 1000*v[T], True,CheckSubCases[game]];  (*Introducing some upper bounds for non-zero-monotone games *)
    uplwc = Which[gzlvQ === False, (x[#] >= v[{#}]) & /@ T, True, (x[#] >= v[{#}] && x[#] <= upb) & /@ T];
    const01 = Which[callm === True, {x[{#}] + Global`\[Alpha] >=  v[#] & /@ mengen, consets, uplwc} // Flatten, 
        True, {x[{#}] + Global`e - Global`g >=  v[#] & /@ mengen, consets, uplwc} // Flatten];
    const02 = MapThread[Equal, {{Apply[Plus, x[#] & /@ T]}, {v[T]}}];
    const = FlattenAt[Append[const01, const02], Length[const01] + 1];
    var = Which[callm === True, FlattenAt[{Global`\[Alpha], x /@ T}, 2], True, FlattenAt[{Global`e, Global`g, x /@ T}, 3]];
    erg = Which[callm === True, Minimize[Global`\[Alpha], const, var], 
           True, ConstrainedMin[Global`e - Global`g, const, var]];
    Which[sil === False, Print["erg=", erg], True, True];
    {alpres,intres} = {erg[[1]],x /@ T /. erg[[2]]};
    Which[(erg[[1]]===-\[Infinity])===True,Return[storeres[[2]]];Break[],True,True];
    oldres = intres; 
    oldalp = alpres;
    alterg = {alpres,intres} === storeres;
    exc = (v[#] - x[#]) /. erg[[2]] & /@ mengen;
    pos = Position[exc, erg[[1]]];
    uniqsets = Delete[mengen, pos];
    bnewsets = Extract[mengen, pos];
   (* Checking now if the collection of sets obtained from the LP has positive dual variables.*)
    balconst = Which[callm===True, x[#]  + (Global`\[Alpha] /. erg[[2, 1]]) >= v[#] & /@ bnewsets//Simplify,
                 True,x[#]  + (Global`e - Global`g /. Take[erg[[2]],2]) >= v[#] & /@ bnewsets//Simplify];
    zf = Apply[Plus, x /@ T];              
    solcoef = SolveDual[zf, balconst, x /@ T];
    t1 = Table[i, {i, Length[solcoef[[2]]]}];
    balcoef = Global`y /@ t1 /. solcoef[[2]];
    zerocfQ = balcoef === Table[0, {i, Length[t1]}];
    balpos = Which[zerocfQ === True,Drop[Position[balcoef, 0],1],True,Position[balcoef, 0]];
 (*All sets with zero weights will be deleted from the collection.*) 
    balsets = Which[zerocfQ === True, Take[bnewsets,1], True,Delete[bnewsets, balpos]];   
    nonbal = Extract[bnewsets, balpos];    (*The sets with zero weights will be appended to sets that constitutes inequalities *)
    nsets = FlattenAt[Append[uniqsets, nonbal], Length[uniqsets] + 1];
    const03 = Which[callm === True, x[{#}] + (Global`\[Alpha] /. erg[[2, 1]]) == v[#] & /@ balsets // Simplify, 
                True, x[{#}] + (Global`e - Global`g /. Take[erg[[2]], 2]) == v[#] & /@ balsets // Simplify];
    solcon = Which[ineq === True, (*Trying to specify the kernel range inside of the Least core. Default value is false *)
                                  (* On a powerful machine you can replace KernelVertices[] by InequalitySolve[] *)
               Which[callm === True, KernelVertices[game], (* InequalitySolve[Simplify[const /. erg[[2, 1]]], x /@ T] *) 
                True, KernelVertices[game]], (*InequalitySolve[Simplify[const /. Take[erg[[2]], 2]], x /@ T] *) 
              True, True];
    const04 = Union[Flatten[{consets, const03}]];
Which[(alterg && (counter>3)) === False, newcounter = counter+1;
  Which[nucval===False,
    Which[KernelImputationListQ[game, intres]  === True, 
          Which[ineq === False,intres, True,{intres,{solcon}}], 
     True, 
       Which[Length[pos] === 0, 
                         Which[ineq === False, intres, True, {intres,{solcon}}], 
            True, CallLP[game, nsets, const04, {oldalp,oldres}, callm, sil, ineq, nucval,newcounter]]],
    True,newcounter = counter+1;(* If nucval is true, the recursion stops if the set of new equal constraints is empty. *) 
        Which[(KernelImputationListQ[game, intres] && Length[balsets]===0 ) === True, 
          Which[ineq === False, intres, True, {intres,{solcon}}], 
          True, 
         Which[Length[pos] === 0, Which[ineq === False, intres, True,{intres,{solcon}}], 
        True, CallLP[game, nsets, const04, {oldalp,oldres}, callm, sil, ineq, nucval,newcounter]]]],
  True,
     Which[KernelImputationListQ[game,intres]===False,
             Which[callm ===False, DisplayMessageNotFound;{intres,{Global`e - Global`g, const, var}},
                   True,DisplayMessageNotFound;{intres,{Global`\[Alpha], const, var}}],
           True,intres]
    ]
]


CheckSubCases[game_]:=Module[{prquo,chquo,sneqsmc,thrpr,possm,sumneqsmc,smcq,smcquo,minprop},
   prquo = Apply[Plus, ProperAmount[game]]/(Length[T]*v[T]);
   chquo = prquo >= 1;
   thrpr= Max[ProperAmount[game]] - v[T];
   sneqsmc= Apply[Plus, SmallestContributionVector[game]*(-1)];
   sumneqsmc = sneqsmc-v[T];
   smcq = sumneqsmc/v[T];
   smcquo = sumneqsmc/(Length[T]*v[T]);
   possm = Min[SmallestContributionVector[game]*(-1)]/Length[T];
   minprop = Min[ProperAmount[game]];
   Which[chquo === True, 
             Which[(smcquo > thrpr), 
                     Which[sumneqsmc > 1,Min[Max[smcquo,prquo],v[T]],
                     True,Min[thrpr,v[T]]],
                  True,Which[sumneqsmc > 1, 
                                      Which[thrpr >= minprop,
                                          Which[smcquo >= minprop,Max[thrpr,v[T]],
                                          True,minprop],
                                        True, Max[thrpr,v[T]]], 
                                      True,Min[thrpr,v[T]]](*minprop *) 
                 ],
        True, Which[(smcquo > thrpr) === True,Min[prquo,v[T]],
                     True, Min[minprop,v[T]]]
        ]
]





OldcallLP[game_, mengen_List, consets_:{}, storeres_:{},callm_, sil_, ineq_,lexival_,counter_:1] := 
  Module[{indratco,const01,const02,const,var,erg,alpres,intres,oldalp,alterg,exc,pos,uniqsets,bnewsets,balconst,zf,solcoef,t1,balcoef,zerocfQ,balpos,balsets,nonbal,nsets,const03,solcon,const04},
    indratco = Which[lexival ===True, (x[#] >= v[{#}] && x[#] <= v[T]) & /@ T,True,x[#] <= v[T] & /@ T];
    const01 = Which[callm === True, {x[{#}] + Global`\[Alpha] >=  v[#] & /@ mengen, consets,indratco} // Flatten, 
                    True, {x[{#}] + Global`e - Global`g >=  v[#] & /@ mengen, consets,indratco} // Flatten];
    const02 = MapThread[Equal, {{Apply[Plus, x[#] & /@ T]}, {v[T]}}];
    const = FlattenAt[Append[const01, const02], Length[const01] + 1];
    var = Which[callm === True, FlattenAt[{Global`\[Alpha], x /@ T}, 2], 
          True, FlattenAt[{Global`e, Global`g, x /@ T}, 3]];
    erg = Which[callm === True, Minimize[Global`\[Alpha], const, var], 
           True, ConstrainedMin[Global`e - Global`g, const, var]];
    Which[sil === False, Print["erg=", erg], True, True];
    {alpres,intres} = {erg[[1]],x /@ T /. erg[[2]]};
    Which[(erg[[1]]===-\[Infinity])===True,Return[storeres[[2]]];Break[],True,True];
    oldalp = alpres;
    oldres = intres; 
    alterg = {alpres,intres} === storeres;
    exc = (v[#] - x[#]) /. erg[[2]] & /@ mengen;
    pos = Position[exc, erg[[1]]];
    uniqsets = Delete[mengen, pos];
    bnewsets = Extract[mengen, pos];
    balconst = Which[callm===True, x[#]  + (Global`\[Alpha] /. erg[[2, 1]]) >= v[#] & /@ bnewsets//Simplify,
                 True,x[#]  + (Global`e - Global`g /. Take[erg[[2]],2]) >= v[#] & /@ bnewsets//Simplify];
    zf = Apply[Plus, x /@ T];
    solcoef = SolveDual[zf, balconst, x /@ T];
    t1 = Table[i, {i, Length[solcoef[[2]]]}];
    balcoef = Global`y /@ t1 /. solcoef[[2]];
    zerocfQ = balcoef === Table[0, {i, Length[t1]}];
    balpos = Which[zerocfQ === True,Drop[Position[balcoef, 0],1],True,Position[balcoef, 0]];
    balsets = Which[zerocfQ === True, Take[bnewsets,1],True,Delete[bnewsets, balpos]];
    nonbal = Extract[bnewsets, balpos];
    nsets = FlattenAt[Append[uniqsets, nonbal], Length[uniqsets] + 1];
    const03 = Which[callm === True, x[{#}] + (Global`\[Alpha] /. erg[[2, 1]]) == v[#] & /@ balsets // Simplify, 
                True, x[{#}] + (Global`e - Global`g /. Take[erg[[2]], 2]) == v[#] & /@ balsets // Simplify];
    solcon = 
    Which[ineq === True, 
      Which[callm === True, 
        InequalitySolve[Simplify[const /. erg[[2, 1]]], x /@ T], True, 
        InequalitySolve[Simplify[const /. Take[erg[[2]], 2]], x /@ T]], True, True];
    const04 = Union[Flatten[{consets, const03}]];
  Which[(alterg && (counter>3)) === False, 
      Which[lexival===False,newcounter = counter+1;
          Which[(PreKernelQ[game,intres]) === True, If[ineq === False, intres, {solcon, intres}], 
                 True, OldcallLP[game, nsets, const04, {oldalp,oldres}, callm, sil, ineq, lexival, newcounter]],
      True, newcounter = counter+1;
           Which[(KernelImputationListQ[game,intres]  && Length[balsets]===0) === True, If[ineq === False, intres, {solcon, intres}], 
                 True, OldcallLP[game, nsets, const04, {oldalp,oldres}, callm, sil, ineq, lexival, newcounter]]
      ],
    True, Which[PreKernelQ[game,intres]===False,
               Which[callm ===False, DisplayMessageNotFound;{intres,{Global`e - Global`g, const, var}},
                    True,DisplayMessageNotFound;{intres,{Global`\[Alpha], const, var}}],
           True,intres]
   ]
]

DisplayMessageNotFound:=(Print["Probably No Kernel/Nucleolus solution found!"];
                         Print["Use the option CallMinimize->True"];
                         Print["to find another solution."]

);





(* Nucleolus/Kernel section ends *)


(* Kernel section starts *)

SmallestContributionVector[game_]:=Table[SmallestContribution[game,i],{i,1,\
Length[T]}]

SmallestContribution[game_,i_]:=
  Min[(v[#]-v[DeleteCases[#,i]])&/@Take[ W[i],-(Length[W[i]]-1)]]


Scrb[game_,i_]:= UtopiaPayoff[game,i] + (1/Length[T]) (v[T] - \
Apply[Plus,UtopiaVector[game]])

ScrbSolution[game_]:= (Scrb[game,#] & ) /@ T

(* Computing the Kernel by a convergence algorithm due to Maschler see Stearns (1968) or U. Faigle, W. Kern and J. Kuipers (1998)*)


FindPreKernelSolution[game_, payoff_List, opts___?OptionQ]:=Module[{prec,reclim,setprk,sil,res},
       Off[$RecursionLimit::reclim]; (*Swichting off all warnings that reclim is exceeded *)
       Off[General::stop];
       Off[MapThread::mptd];
       Off[MapThread::list];
       Off[ReplaceAll::reps];
       Off[Extract::psl];
       Off[Part::partw];
       Off[Part::pspec];
       Off[Part::partd];
       Off[ReplacePart::psl];
       prec = DigitPrecision /. {opts} /. Options[FindPreKernelSolution];
       rattol = RationalTol /. {opts} /. Options[FindPreKernelSolution];
       reclim = SetRecursionLimit /. {opts} /. Options[FindPreKernelSolution];
       sil = Silent /. {opts} /. Options[FindKernelSolution];
       setprk = PRK;
       res = Block[{$RecursionLimit = reclim},
             Which[Depth[payoff] == 3, KernelSolution[game,#,prec,rattol,setprk,sil] &/@ payoff,
                   Depth[payoff] == 2, KernelSolution[game,payoff,prec,rattol,setprk,sil],
                   True, DisplayMessFindKernel[payoff]]];
       On[$RecursionLimit::reclim]; (*Swichting on all warnings that have been turned off *)
       On[General::stop];
       On[MapThread::mptd];
       On[MapThread::list];
       On[ReplaceAll::reps];
       On[Extract::psl];
       On[Part::partw];
       On[Part::pspec];
       On[Part::partd];
       On[ReplacePart::psl];
       res = Which[Depth[res] == 2, res,
             Depth[res] == 3, If[Last[res] === 1, DisplayMessReclim[reclim]; First[res], res], 
             True, res];
       Which[Depth[res] == 2, Flatten[res],
             Depth[res] == 3, SolutionSelection[game,res],
             Depth[res] == 4, SolutionSelection[game,#]&/@ Table[res[[i]],{i,1,Length[res]}],
             True, res
       ]
]


FindKernelSolution[game_, payoff_List, opts___?OptionQ]:=Module[{prec,reclim,setker,sil,res},
       Off[$RecursionLimit::reclim]; (*Swichting off all warnings that reclim is exceeded *)
       Off[General::stop];
       Off[MapThread::mptd];
       Off[MapThread::list];
       Off[ReplaceAll::reps];
       Off[Extract::psl];
       Off[Part::partw];
       Off[Part::pspec];
       Off[Part::partd];
       Off[ReplacePart::psl];
       prec = DigitPrecision /. {opts} /. Options[FindKernelSolution];
       rattol = RationalTol /. {opts} /. Options[FindKernelSolution];
       reclim = SetRecursionLimit /. {opts} /. Options[FindKernelSolution];
       sil = Silent /. {opts} /. Options[FindKernelSolution];
       setker = KER;
       res = Block[{$RecursionLimit = reclim},
             Which[Depth[payoff] == 3, KernelSolution[game,#,prec,rattol,setker,sil] &/@ payoff,
                   Depth[payoff] == 2, KernelSolution[game,payoff,prec,rattol,setker,sil],
                   True, DisplayMessFindKernel[payoff]]];
       On[$RecursionLimit::reclim]; (*Swichting on all warnings that have been turned off *)
       On[General::stop];
       On[MapThread::mptd];
       On[MapThread::list];
       On[ReplaceAll::reps];
       On[Extract::psl];
       On[Part::partw];
       On[Part::pspec];
       On[Part::partd];
       On[ReplacePart::psl];
       res = Which[Depth[res] == 2, res,
             Depth[res] == 3, If[Last[res] === 1, DisplayMessReclim[reclim]; First[res], res], 
             Depth[res] == 4, If[Last[#] === 1, DisplayMessReclim[reclim]; First[#], #]&/@ res, 
             True, res];
       Which[Depth[res] == 2, Flatten[res],
             Depth[res] == 3, SolutionSelection[game,res],
             Depth[res] == 4, SolutionSelection[game,#]&/@ Table[res[[i]],{i,1,Length[res]}],
             True, res
       ]
]

SolutionSelection[game_,payoff_]:= Module[{bool,pos,lsg},
         bool = KernelImputationListQ[game,payoff];
         pos = Position[bool,True];
         lsg = If[Length[pos]===0,payoff,Extract[payoff,pos]];
      Which[Length[lsg]===1,Flatten[lsg],True,lsg]
]

KernelSolution[game_, payoff_, prec_,rattol_,setk_,silent_,payold_:{}] := Module[{payvec, pold=payold,approx,ratnum},
    payvec = ConvergenceToKernel[game, payoff];
    pold = payoff;
    approx = N[payvec, prec];
    Which[silent === False,Print["ConvergingQ: ",N[pold,prec]],True,True];
    ratnum = Rationalize[approx,rattol];
    If[Depth[payvec]<=3, 
      Which[PRK === setk,
       Which[Apply[Or,PreKernelQ[game, {pold,ratnum}]] === False, 
           KernelSolution[game, payvec, prec, rattol,setk,silent,ratnum],
           True, Union[{payvec,ratnum}]],
      KER === setk, Which[Apply[Or,KernelImputationListQ[game, {pold,ratnum}]] === False, 
      KernelSolution[game, payvec, prec, rattol,setk,silent,ratnum],
      True, Union[{payvec,ratnum}]],True, Union[{payvec,ratnum}]],
      {pold,1}
     ]
    ]



ConvergenceToKernel[game_, payoff_] := 
  Module[{survec, surpl, deltavec, soldel, maxdel, posmdel, minpos, 
      dir, pli, plj, plpair, extr, trfij, numval, travec, nwpay, mp},
    survec = AllMaxSurpluses[game, payoff][[1]];
    surpl = ReplaceAll[#, List -> Subtract] & /@ survec;
    deltavec = Table[- 2 Global`\[Alpha], {i, Length[surpl]}];
    soldel = 
      MapThread[
        Flatten[Rationalize[Solve[#1 == #2, Global`\[Alpha]]]] &, {surpl, 
          deltavec}];
    solvec = Global`\[Alpha] /. soldel;
    maxdel = Max[Abs[solvec]];
    posmdel = Position[Abs[solvec], maxdel];
    minpos = Min[posmdel];
    dir = Flatten[
        Table[Table[{i, j}, {j, i + 1, Length[T]}], {i, 1, Length[T]}]];
    pli = fipl[dir];
    plj = secpl[dir];
    plpair = MapThread[List, {pli, plj}];
    extr = Extract[plpair, minpos];
    trfij = Extract[payoff, Outer[List, extr]];
    numval = Flatten[posmdel][[1]];
    travec = {-solvec[[numval]], solvec[[numval]]};
    nwpay = trfij + travec;
    mp = MapThread[List,{nwpay,extr}];
    sol = Rp[Rp[payoff,mp[[1,1]],mp[[1,2]]],mp[[2,1]],mp[[2,2]]]
    ]

Rp[vec_, nwcom_, posi_] := ReplacePart[vec, nwcom, posi]


(* Auxiliary functions to construct the (n (n-1)/2) initial LPs for KernelCalculation[] *)

Tijmenge[i_, j_] := 
    DeleteCases[Cases[ProperCoalitions, {___, i, ___}], {___, j, ___}];

DelRat[i_, j_, Global`\[Delta]_, eps_:0] := 
  x[#] + Global`\[Delta] >= v[#]  & /@ Tijmenge[i, j]



TransferConstraints[game_, i_, j_, eps_:0] := 
    Append[Core[game], {DelRat[i, j, -Global`\[Delta], eps], 
           DelRat[j, i, -Global`\[Delta], eps], 
           delconst}] // Flatten;
(*
DeltaLP[game_, i_, j_, eps_:0] := 
    ConstrainedMax[Global`\[Delta], Union[TransferConstraints[game, i, j, \
eps]], 
      Join[x /@ T, {Global`\[Delta]}]];
*)

DeltaLP[game_, i_, j_, eps_:0, changesolver_] := Module[{},
        Which[changesolver==False, ConstrainedMax[Global`\[Delta], Union[TransferConstraints[game, i, j, eps]], Join[x /@ T, {Global`\[Delta]}]],
              True, Maximize[Global`\[Delta], Simplify[Union[TransferConstraints[game, i, j, eps]]], Join[x /@ T, {Global`\[Delta]}]]]
];

delconst = {Global`\[Delta] >=  0};


pay = x /@ T;

Transfer[y_, T_, d_, i_, j_] := 
  Module[{idmat}, idmat = IdentityMatrix[Length[T]];
    (x /@ T /. Payoff[y]) - d Extract[idmat, i] + d Extract[idmat, j]]

MaxDeltaij[game_, eps_:0,changesolver_] := 
  Module[{anf = T, rest, pi, pj, erg = {}, res}, While[Length[anf] > 1,
      pi = First[anf];
      rest = Drop[anf, 1];
      While[Length[rest] > 0,
        pj = First[rest];
        erg = Append[erg, DeltaLP[game, pi, pj, eps,changesolver]];
        rest = Drop[rest, 1];
        ];
      anf = Drop[anf, 1];];
    res = erg]

TijLoop[game_] := 
  Module[{anf = T, rest, pi, pj, menge = {}, mengsys}, While[Length[anf] > 1,
      pi = First[anf];
      rest = Drop[anf, 1];
      While[Length[rest] > 0,
        pj = First[rest];
        menge = Flatten[
        Append[menge, Union[Tijmenge[pi, pj], Tijmenge[pj, pi]]], 0];
        rest = Drop[rest, 1];
        ];
      anf = Drop[anf, 1];];
    mengsys = menge]


NewIneq[ineq_, param_, liste_] := 
  Table[ineq[[i]] /. {lhs_ <= rhs_ :> 
          lhs <= rhs + param[[i]] /; rhs == Min[liste]}, {i, Length[param]}]

SortConstraints[ineq_, para_] := 
  Module[{pospattern, extpos}, 
    pospattern = Position[ineq, Cases[ineq, _. + para, {2}][[1]], {2}];
    extpos = Outer[List, Table[pospattern[[i, 1]], {i, Length[pospattern]}]];
    Extract[ineq, extpos]]


FeasibleConstraints[ineq_, delta_,changesolver_] := 
  Module[{delexc = Sort[delta], creatpara, redpara, paramet, nebenbed, fineq, \
      feasbineq = {}, allineq, varb, sol, resl, setofineq},
    newungl = Drop[ineq, -1];
    creatpara = Table[Table[a[i], {Length[newungl]}], {i, Length[delexc]}];
    redpara = Union[Flatten[creatpara]];
    While[Length[delexc] > 0, paramet = Take[creatpara, 1][[1]];
      nebenbed = NewIneq[newungl, paramet, delexc];
      fineq = SortConstraints[nebenbed, paramet[[1]]];
      feasbineq = Append[feasbineq, fineq] // Flatten;
      creatpara = Drop[creatpara, 1];
      delexc = Drop[delexc, 1];
      ];
    allineq = Simplify[Flatten[Append[feasbineq, Last[ineq]]]];
    varb = Append[x /@ T, redpara] // Flatten;
    sol = Which[changesolver == False, ConstrainedMin[Apply[Plus, redpara], allineq, varb],
                True, Minimize[Apply[Plus, redpara], allineq, varb]];
    resl = x /@ T /. Take[sol[[2]],Length[T]];
    setofineq = allineq /. Take[Flatten[Take[sol, 2]], -Length[redpara]];
    {resl, setofineq}
 ]




NewConstraints[sets_, constraints_, del_, eps_:0] := 
  Module[{Tijsys, delta, dij = del, mengsys = sets, exccon, 
      const = constraints},
    While[Length[dij] > 0,
      delta = Take[dij, 1][[1]];
      Tijsys = Take[mengsys, 1];
      exccon = v[#] - x[#] <= -delta & /@ Tijsys[[1]];
      const = Append[const, exccon];
      dij = Drop[dij, 1];
      mengsys = Drop[mengsys, 1]]; const]


(* Auxiliary functions to verify balancedness of the excess vector *)


CheckBalancedness[game_, pay_] := Module[{ balancedQ},
    balancedQ = MaxExcessBalanced[newgame, pay][[1]]
    ]



NewCheckBalancedness[game_, pay_, w_:v] := 
  Module[{balancedQ}, 
    balancedQ = KernelImputationQ[newgame, pay, Global`w]
    ]



AssgPay[liste_] := 
    Table[MapThread[Rule, {Map[x, T], liste[[i]]}], {i, Length[liste]}];

MaxExcess[sijset_, payoff_] := 
    Module[{maxex}, 
      maxex = Table[
          Table[Max[((v[#] - x[#]) & /@ sijset[[i]]) /. payoff[[k]]], {i, 
              Length[sijset]}], {k, Length[payoff]}]];

MaxSurplus[game_, pi_, pj_, payoff_] := 
    Module[{payass}, 
      payass = Which[Depth[payoff]==3, MapThread[Rule,{x /@ T,#}]& /@ payoff, 
                     Depth[payoff]==2, MapThread[Rule,{x /@ T,payoff}],
                     True, Print["The input 'payoff' is not a list."];Break[]];
      Which[Depth[payass] == 5,Max[ReplaceAll[(v[#] - x[#]) & /@ TIJsets[pi,pj],#]] &/@ payass,
            Depth[payass] == 4,Max[ReplaceAll[(v[#] - x[#]) & /@ TIJsets[pi,pj],payass]],
            True, Print["Wrong data format."];Break[]]
      ] 


MaxSurpluses[game_, payoff_] := 
    Module[{dir,pli,plj,maxpi,maxpj,res}, 
      dir = Flatten[Table[Table[{i,j},{j,i+1,Length[T]}],{i,1,Length[T]}]];
      pli = fipl[dir];
      plj = secpl[dir];
      maxpi = MapThread[MaxSurplus[game,#1,#2,payoff]&,{pli,plj}];
      maxpj = MapThread[MaxSurplus[game,#1,#2,payoff]&,{plj,pli}];
      MapThread[List,{maxpi,maxpj}]
     ]

AllMaxSurpluses[game_,payoff_,opts___?OptionQ]:= Module[{ausz,sol,mgij,mgji,ergb},
     dispmat = DisplayMatrixForm /. {opts} /. Options[AllMaxSurpluses];    
     ausz = Which[Depth[payoff]==2,{payoff},
                  Depth[payoff]==3,payoff,
                  True, Print["The input 'payoff' is not a list."];Break[]];
     ergb = MaxSurpluses[game,#] &/@ ausz;
     Which[dispmat == False, ergb, True, DisplayResult[ergb]]
]

DisplayResult[payoff_]:= Module[{surpl,dir,pli,plj,setij,setji,nwij,mgij},
    surpl = payoff; 
    dir = Flatten[Table[Table[{i,j},{j,i+1,Length[T]}],{i,1,Length[T]}]];
    pli = fipl[dir];
    plj = secpl[dir];
    setij = MapThread[List,{pli,plj}];
    setji = MapThread[List,{plj,pli}];
    nwij = MapThread[List,{setij,setji}];
    mgij = Map[Global`sij,#] &/@ nwij;
    MatrixForm[PrependTo[surpl,mgij]]
]




fipl[list_, storeli_:{}] := Module[{rest, init = storeli},
    AppendTo[init, First[list]];
    rest = Drop[list, 2];
    If[Length[rest] !=  0, fipl[rest, init], init]
    ]

secpl[list_, storeli_:{}] := Module[{rest, init = storeli},
    AppendTo[init, list[[2]]];
    rest = Drop[list, 2];
    If[Length[rest] !=  0, secpl[rest, init], init]
    ]


KernelImputationQ[game_, payoff_, Global`w_:v] := 
  Module[{sij, sji, li, assg, maxexij, maxexji, lhs12, lhs22, kerncond1, 
      kerncond2, zeros, checktrue},
  If[Depth[payoff]==2,
    sij = Flatten[Table[Table[Tijmenge[i, j], {j, i + 1, Length[T]}], {i, Length[T]}],1];
    sji = 
      Flatten[Table[
                Table[Tijmenge[j, i], {j, i + 1, Length[T]}], {i, Length[T]}], 1];
    li = If[Length[Dimensions[payoff]] == 1, {payoff}, payoff];
    assg = AssgPay[li];
    maxexij = Rationalize[Flatten[MaxExcess[sij, assg]]];
    maxexji = Rationalize[Flatten[MaxExcess[sji, assg]]];
    lhs12 = (Table[((x[#] - Global`w[{#}] & /@ sji[[i, 1]])[[1]] /. assg)[[1]], {i, Length[sji]}]);
    lhs22 = (Table[((x[#] - Global`w[{#}] & /@ sij[[i, 1]])[[1]] /. assg)[[1]], {i, Length[sij]}]);
    zeros = Table[0, {Length[sij]}];
    kerncond1 = 
      MapThread[
        LessEqual, {Flatten[
            Table[(maxexij[[i]] - maxexji[[i]]) lhs12[[i]], {i,Length[maxexij]}]], zeros}];
    kerncond2 = 
      MapThread[
        LessEqual, {Flatten[
            Table[(maxexji[[i]] - maxexij[[i]]) lhs22[[i]], {i, Length[maxexji]}]], zeros}];
    checktrue = Apply[And, MapThread[And, {Rationalize[kerncond1], Rationalize[kerncond2]}]];
    checktrue, DisplayMess[payoff]
    ]
  ]

DisplayMessFindKernel[payoff_]:=(
       Print["Depth is equal to ",Depth[payoff]];
       Print["Usage: FindKernelSolution[game,payoff]"];
       Print["Input format of the variable 'payoff' is not correct."];
       Print["The variable 'payoff' must be a single payoff vector or a list of payoffs."]
);

DisplayMess[payoff_]:=(
       Print["Depth is equal to ",Depth[payoff]];
       Print["Usage: KernelImputationQ[game,payoff]"];
       Print["Input format of the variable 'payoff' is not correct."];
       Print["The variable 'payoff' must be a single payoff vector."];
       Print["Use the function KernelImputationListQ[game,payoff]"];
       Print["to check a list of payoff vectors."]
);

DisplayMessReclim[reclimval_]:=(
       Print["Warning:: Recursion limit exceeds threshold value:", reclimval];
       Print["Use the function KernelImputationListQ[game,payoff]"];
       Print["to check if the result is a kernel solution."]       

);

KernelImputationListQ[game_, payoff_] := Module[{kerimp,imp},
    Which[Depth[payoff] == 3, 
            kerimp = KernelImputationQ[game, #] &/@ payoff;
            imp = CheckImputations[game, #] & /@ payoff;
            MapThread[And,{kerimp,imp}],
          Depth[payoff] == 2, 
            kerimp = KernelImputationQ[game,payoff];
            imp = CheckImputations[game,payoff];
            Apply[And,{kerimp,imp}],
          True, DisplayMessage[payoff]
    
   ]
]



ImputationQ[game_,payoff_]:= Module[{},
   Which[Depth[payoff] == 3, CheckImputations[game, #] & /@ payoff,
         Depth[payoff] == 2, CheckImputations[game,payoff],
         True, DisplayMessage[payoff]
   ]
]


CheckImputations[game_,payoff_]:= Module[{},
    imp = Imputations[game] /. MapThread[Rule, {x[#] & /@ T, payoff}];
    Apply[And, imp]
]



DisplayMessage[payoff_]:=( 
       Print["Depth is equal to ",Depth[payoff]];
       Print["Usage: KernelImputationListQ[game,payoff] and PreKernelQ[game,payoff]"];
       Print["Input format of the variable 'payoff' is not correct."];
       Print["The variable 'payoff' must be a list of payoff vectors or a single payoff vector."]

);


PreKernelQ[game_, payoff_] :=Module[{graval}, 
       graval = v[T];
       Which[Depth[payoff] == 2, MapThread[And,{{Apply[Plus, payoff] == graval},MaxExcessBalanced[game, payoff]}], 
             Depth[payoff] == 3, MapThread[And,{(Apply[Plus, #] == graval) & /@ payoff,MaxExcessBalanced[game, payoff]}],
             True, DisplayMessage[payoff]
        ]
     ]

(* User interface to calculate kernel elements of a Tu-game. *)


KernelCalculation[game_, opts___?OptionQ] := 
 Module[{changeps,dispres, epsi1, fieps, notzeromono, firstcrit,kernres, epsilon, zmonoQ, kernQ, avcQ, orggame, orgval},
  callm = CallMaximize /. {opts} /. Options[KernelCalculation];
  changeps = ChangeInternalEps /. {opts} /. Options[KernelCalculation];
  dispres = DisplayAllResults /. {opts} /. Options[KernelCalculation];
  epsi1 = EpsilonValue  /. {opts} /. Options[KernelCalculation];
  notzeromono = SetGameToNonZeroMonotonic /. {opts} /. Options[KernelCalculation]; 
   If[Depth[epsi1]==1, 
    ClearProtected[Global`w];
    MapThread[Set, {{Global`w[#]}, {v[#]}}] & /@ Subsets[T];
    SetAttributes[Global`w, Protected];
    orgval = Rationalize[v[#] & /@ Subsets[T]]; 
(* ZeroMonotonQ[] changes the coalitional values. orgval stores the original values.  *)
    fieps= FirstCriticalVal[game][[1,2]];
    epsilon = If[epsi1 < fieps,fieps,epsi1];
    zmonoQ = ZeroMonotoneQ[game];
    orggame = DefineGame[T,orgval]; (* Reconstruction of the original game. *)
    kernQ = CoreQ[orggame];
    avcQ = AvConvexQ[orggame];
    If[epsilon != 0, (* For zero-monotonic games with an nonempty core we can use KernelPoints[], 
                        otherwise we use the function Kernel[].
                        KernelPoints[] can run into infinite loops for 
                        non zero-monotonic games.
                        This function seems to be stable for non zero-monotonic games. *) 
                    If[ (zmonoQ || kernQ) == True, 
                           Print["Game is zero-monotone? ",zmonoQ];
                           Print["Core is nonempty? ",kernQ];
                           Print["Game is either zero-monotonic or has nonempty core"];
                           kernres = KernelPoints[orggame, {},callm,changeps, dispres, EpsilonValue -> epsilon, opts],
                      Print["Game is zero-monotone? ",zmonoQ];
                      Print["Core is nonempty? ",kernQ];
                      Print["Game is not zero-monotonic and has empty core"];
                      kernres = KernelSubCallZero[orggame, callm, changeps, dispres, EpsilonValue -> epsilon, SetGameToNonZeroMonotonic -> notzeromono]],
     If[ (zmonoQ || kernQ) == False,
       Print["Game is zero-monotone? ",zmonoQ];
       Print["Core is nonempty? ",kernQ];
       Print["Game is not zero-monotonic and has empty core"];
       kernres = KernelSubCallZero[orggame, callm, changeps, dispres, EpsilonValue -> epsilon, SetGameToNonZeroMonotonic -> notzeromono],
         If[ kernQ == False, 
          Print["Game is zero-monotone? ",zmonoQ];
          Print["Core is nonempty? ",kernQ];
          Print["Core is empty"];
          kernres = KernelSubCallEmpty[orggame,callm, changeps, dispres, EpsilonValue -> epsilon, SetGameToNonZeroMonotonic -> notzeromono],
           If[avcQ == True, 
            Print["Game is average-convex? ",avcQ];
            kernres = KernelSubCall[orggame,callm, changeps, dispres, EpsilonValue -> epsilon, SetGameToNonZeroMonotonic -> notzeromono],
            Print["Game is average-convex? ",avcQ];
            kernres = KernelSubCall[orggame,callm, changeps, dispres, EpsilonValue -> epsilon, SetGameToNonZeroMonotonic -> notzeromono]
           ]
         ]
      ]
     ], DisplayRemark[epsilon]
   ];
   DefineGame[T,orgval]; (* Reconstruction of the original game. *)
   kernres
  ]
 


DisplayRemark[eps_] := (
      Print["Depth is equal to ",Depth[eps]];
      Print["Usage: Kernel[game,options]"];
      Print["and KernelCalculation[game,options]"];
      Print["The variable 'eps' has not the correct input format."];
      Print["The variable 'eps' must be a rational number and not a list."];
      Print["Use for instance the function FirstCriticalVal[] to find out such a number."]);



KernelSubCallEmpty[game_,callm_,changeps_,dispres_,opts___?OptionQ] := 
     Module[{epsi1,notzeromono,critical},
          epsi1 = EpsilonValue  /. {opts} /. Options[KernelSubCallEmpty];
          notzeromono = SetGameToNonZeroMonotonic /. {opts} /. Options[KernelSubCallEmpty]; 
          critical = Which[changeps == False, epsi1, True, SecondCriticalVal[game][[1, 2]]];
          kernres = KernelPoints[game, {}, callm, changeps, dispres, EpsilonValue -> critical, SetGameToNonZeroMonotonic -> notzeromono] 


 ]

KernelSubCall[game_,callm_,changeps_,dispres_,opts___?OptionQ] := 
     Module[{epsi1,notzeromono,critical},
          epsi1 = EpsilonValue  /. {opts} /. Options[KernelSubCall];
          notzeromono = SetGameToNonZeroMonotonic /. {opts} /. Options[KernelSubCall]; 
          critical = Which[changeps == False, epsi1, True, FourthCriticalVal[game][[1, 2]]];
          kernres = KernelPoints[game, {}, callm, changeps, dispres, EpsilonValue -> critical, SetGameToNonZeroMonotonic -> notzeromono] 


 ]


KernelSubCallZero[game_,callm_,changeps_,dispres_,opts___?OptionQ] := 
     Module[{epsi1,notzeromono,critical,kernres,kernsol},
          epsi1 = EpsilonValue  /. {opts} /. Options[KernelSubCallZero];
          notzeromono = SetGameToNonZeroMonotonic /. {opts} /. Options[KernelSubCallZero]; 
          critical = Which[changeps == False, epsi1, True, StarCriticalVal[game][[1, 2]]];
(* 
The function KernelPoints[] is not stable for non zero-monotonic games.
We comment the function KernelPoints[] out and rely on the stable 
function Kernel[] to avoid infinite loops.
*)
(*
          kernres = KernelPoints[game, {}, critical]
*)
       Which[notzeromono == False,
             kernsol= Take[Kernel[game, DisplayAllResults -> True, EpsilonValue -> critical],3];
             kernres = Append[kernsol,kernsol[[1]]],
            True, KernelPoints[game, {}, callm, changeps, dispres, EpsilonValue -> critical, SetGameToNonZeroMonotonic -> notzeromono]
         ]
 ]



(* 
Main functions to determine kernel elements according to the algorithm 
as described in Meinhardt(2002). Recursive procedure. 
*)

KernelPoints[game_, storepay_:{}, callm_, changeps_, dispres_, opts___?OptionQ] := 
  Module[{epsi1, notzeromono, ruf, zwerg, initialpay = storepay, balanc, folg, del, del2, alloc, po, 
      maxsets, tijsets, mengsys, exccon, reslofSt2, constofSt2, const = {}, sol, newconst, 
      returnvalue, objfunc, epsilon, leastgame, kernelsol},
    epsi1 = EpsilonValue  /. {opts} /. Options[KernelPoints];
    notzeromono = SetGameToNonZeroMonotonic /. {opts} /. Options[KernelPoints]; 
(*    Clear[epsilon]; *)
    epsilon = epsi1;
    newgame = EpsValue[game, epsi1];
    ruf = MaxDeltaij[newgame, epsi1,callm];
    folg = (Length[T](Length[T] - 1))/2;
    zwerg = Table[Take[ruf[[i, 2]], Length[T]], {i, folg}];
    initialpay = 
      FlattenAt[Append[initialpay, Union[(x /@ T) /. zwerg]], 
        Length[initialpay] + 1];
    balanc = MaxExcessBalanced[newgame, (x /@ T) /. zwerg];
    del = Global`\[Delta] /. 
        Table[Take[MaxDeltaij[newgame, epsi1,callm][[i, 2]], -2], {i, folg}];
    tijsets = TijLoop[newgame];
    po = Table[
        Position[
          Flatten[ExcessPayoff[
              newgame, (x /@ T) /. zwerg[[i]]]], -del[[i]]], {i, 
          Length[del]}];
    maxsets = Table[Extract[Subsets[T], po[[i]]], {i, Length[po]}];
    mengsys = 
      Table[Intersection[maxsets[[i]], tijsets[[i]]], {i, Length[maxsets]}];
    const = NewConstraints[mengsys, const, del, epsi1];
    const = Flatten[Join[Append[const, Apply[Plus, x /@ T] <= v[T]]]];
    reddel = Sort[-Union[del]];
    {reslofSt2, constofSt2} = FeasibleConstraints[const, reddel,callm];
    initialpay = Append[initialpay,reslofSt2];
    unpay = Union[initialpay];
    const = Simplify[Union[constofSt2]];
    sol = Which[callm == False, ConstrainedMax[Apply[Plus, x /@ T], const, x /@ T],
                True, Maximize[Apply[Plus, x /@ T], const, x /@ T]];
    alloc = x /@ T /. sol[[2]];
    objfunc = Apply[Plus, x /@ T];
    returnvalue = Which[notzeromono == False, NewCheckBalancedness[newgame, alloc, Global`w], 
                        True, (PreKernelQ[newgame, alloc][[1]] || NewCheckBalancedness[newgame, alloc, Global`w])];
    If[returnvalue == True,
      Print["A Kernel solution is: ",MapThread[Rule,{x /@ T,alloc}]];
      Which[dispres == False, {alloc,unpay}, True, {alloc, objfunc, const, del-epsi1, unpay}],
      Print["At least one player outweighs another player."]; 
      Print["A further calculation step is required.\n", sol];

      If[epsi1 != 0,
        Print["Solution is not balanced."];
        Print["The solution found may be a Kernel element."];
        Print["Check solution with the function KernelImputationQ or MaxExcessBalanced."];
        kernelsol = KernelPoints[newgame, initialpay,callm,changeps,dispres,EpsilonValue -> 0, SetGameToNonZeroMonotonic -> notzeromono];
         kernelsol,
        epsilon = FirstCriticalVal[newgame][[1, 2]];
        If[epsilon == 0,
          epsilon = StarCriticalVal[newgame][[1, 2]];
          leastgame = EpsValue[newgame, epsilon];
           kernelsol = KernelPoints[leastgame, initialpay,callm,changeps,dispres,EpsilonValue -> 0, SetGameToNonZeroMonotonic -> notzeromono];
           kernelsol,
          leastgame = EpsValue[newgame, epsilon];
          kernelsol = KernelPoints[leastgame, initialpay,callm,changeps,dispres,EpsilonValue -> 0, SetGameToNonZeroMonotonic -> notzeromono];
          kernelsol
          ]
        ]
      ]
    ]





(* NewKernel section starts *)
(* Description of the algorithm can be found in section 5 in Meinhardt (2002) *)

Kernel[game_,opts___?OptionQ] := Module[{chopt, dispres, epsilon, coreq,kersol,orgval,callm},
  dispres = DisplayAllResults  /. {opts} /. Options[Kernel];
  eps = EpsilonValue  /. {opts} /. Options[Kernel];
  callm = CallMaximize /. {opts} /. Options[Kernel];
   orgval = Rationalize[v[#] & /@ Subsets[T]]; 
(* EpsValue[] changes the coalitional values. orgval store the original values.  *)
   If[Depth[eps]==1,
(*     epsilon = eps;*)
     coreq = CoreQ[game];
     If[eps == 0, 
       If[coreq == False, kersol=KernelSubCallFir[game,dispres,callm],kersol = KernelSubCallSec[game,dispres,callm]
      ], kersol=KernelSubCallThir[game,eps,dispres,callm]
   ], DisplayRemark[eps]    
  ];
  DefineGame[T,orgval]; (* Reconstruction of the original game. *)
  kersol
]

KernelSubCallFir[game_,dispres_,callm_]:= 
Module[{firstcrit,seccrit,newgame,nb,alldelvar,dvar,objfunc,var,solut,kersol,tra},
        Print["Game has empty core"];
        firstcrit = FirstCriticalVal[game][[1,2]];
        newgame = EpsValue[game, firstcrit];
        {nb,alldelvar,dvar} = AllConstraints[newgame];
        objfunc= Apply[Plus,dvar];
        var= Union[x /@ T, alldelvar];
        solut = Which[callm===False,ConstrainedMax[objfunc,nb,var],True,Maximize[objfunc,nb,var]];
        kersol=(x /@ T) /.Take[solut[[2]],Length[T]];
        tra= dvar /. Take[solut[[2]],-Length[alldelvar]];
        Which[dispres == False, kersol, True, {kersol,objfunc,nb,var,tra}]
]


KernelSubCallSec[game_,dispres_,callm_]:= 
Module[{nb,alldelvar,dvar,objfunc,var,solut,kersol,tra},
             Print["Game has nonempty core"];
             {nb,alldelvar,dvar} = AllConstraints[game]; 
             objfunc= Apply[Plus,dvar];
             var= Union[x /@ T, alldelvar];
             solut = Which[callm===False,ConstrainedMax[objfunc,nb,var],True,Maximize[objfunc,nb,var]];
             kersol=(x /@ T) /.Take[solut[[2]],Length[T]];
             tra= dvar /. Take[solut[[2]],-Length[alldelvar]];
             Which[dispres == False, kersol, True, {kersol,objfunc,nb,var,tra}]

]

KernelSubCallThir[game_,eps_,dispres_,callm_]:= 
Module[{newgame,nb,alldelvar,dvar,objfunc,var,solut,kersol,tra},
      newgame = EpsValue[game, eps];
      {nb,alldelvar,dvar} = AllConstraints[newgame];
      objfunc= Apply[Plus,dvar];
      var= Union[x /@ T, alldelvar];
      solut = Which[callm===False,ConstrainedMax[objfunc,nb,var],True,Maximize[objfunc,nb,var]];
      kersol=(x /@ T) /.Take[solut[[2]],Length[T]];
      tra= dvar /. Take[solut[[2]],-Length[alldelvar]];
      Which[dispres == False, kersol, True, {kersol,objfunc,nb,var,tra}]

]





(* Constructing the one large LP for NewKernel[] *)

AllConstraints[game_]:=
  Module[{anf=T,rest,pi,pj,const,conset={},sub,deltaset={},deltavar={},
      newconst,constsys},
      While[Length[anf]>1,
      pi=First[anf];
      rest=Drop[anf,1];
      While[Length[rest]>0,pj=First[rest];
        const=TransferConstraints[game,pi,pj];
        sub=MapThread[
            Rule,{{ Global`\[Delta] }, {Global`\[Delta][pi,pj]}}];
(*        deltaset=Append[deltaset,{Global`\[Delta][pi,pj],Global`\[Delta][pj,pi]}];*)
        deltavar=Append[deltavar,{Global`\[Delta][pi,pj]}];
        newconst=const/.sub;
        conset=Append[conset,newconst];
        rest=Drop[rest,1];
        ];
      anf=Drop[anf,1];];
    constsys=Union[Flatten[conset]];
    {constsys,Flatten[deltavar],Flatten[deltavar]}
    ]

(* NewKernel section ends *)

(* Section starts to determine a Kernel Range from an LP. *)

KernelVertices[game_, opts___?OptionQ] := 
  Module[{callm, sil, epsval, fieps, kerli, sol, obj, const, var, trans,kerQ},
    callm = CallMaximize /. {opts} /. Options[KernelVertices];
    sil = Silent  /. {opts} /. Options[KernelVertices];
    epsval = EpsilonValue /. {opts} /. Options[KernelVertices];
    fieps= FirstCriticalVal[game][[1,2]];
    epsval = If[epsval < fieps, fieps,epsval];
    {sol, obj, const, var, trans} = 
      Which[callm === True,  Kernel[game, CallMaximize -> True, EpsilonValue -> epsval, DisplayAllResults -> True], 
            True,  Kernel[game, CallMaximize -> False, EpsilonValue -> epsval, DisplayAllResults -> True]];
    (*Print["sol1=", sol];*)
    kerli = DetermineAddVertices[{sol}, obj, const, var, T, callm,sil];
    kerQ=KernelImputationListQ[game,kerli];
    posker=Position[kerQ,True];
    Extract[kerli,posker]
    ]


DetermineAddVertices[sol_, obj_, const_, var_, T_, callm_,sil_, zf_:{}] := 
  Module[{nwzf,dures,valzf,boolzf,dualvar,duyp,newineq,nwsol,prmax,lngt,oldres=sol},
    nwzf = Apply[Plus, var];
    dures = SolveDual[nwzf, const, var];
    valzf = dures[[1]];
    boolzf = valzf === zf;
    Which[boolzf === True, Return[oldres]; Break[], True, True];
    dualvar = Global`y /@ Table[i, {i, Length[dures[[2]]]}];
    yset = dualvar /. dures[[2]];
    duyp = Position[yset, 0];
    newineq = Extract[const, duyp];
    nwsol = Which[callm === True, Maximize[nwzf, newineq, var], 
                True, ConstrainedMax[nwzf, newineq, var]];
    prmax = nwsol[[1]];
    Which[(!sil)===True,Print["DualSol: ",{dures[[1]],Delete[dures[[2]],duyp]}];Print["PrimalSol: ",nwsol];,True,True];
    lngt = Length[T];
    kersolQ = Take[var, lngt] /. Take[nwsol[[2]], lngt];
    AppendTo[oldres, kersolQ];
    oldres = Union[oldres];
    Which[(sol === oldres && (2*prmax > valzf)) === True, oldres, True, 
      DetermineAddVertices[oldres, obj, newineq, var, T, callm, sil, valzf]]
]

(* Section ends to determine a Kernel Range from an LP. *)


(* Kernel section ends *)


(* Start mathlink section *)

(*If you have not installed cddmathlink then you should deactivate the \
functions AllVertices and all functions starting with 'Cdd'. *)		
		
		
AllVertices[m_Integer, d_Integer, a_List] := ExternalCall[cddml,CallPacket[0,{m,d,a}]] 

(* Uncomment the following lines under UNIX/LINUX *)
(*
AllVertices[m_Integer, d_Integer, a_String] := ExternalCall[cddmlgmp,CallPacket[0,{m,d,a}]] 
AllVertices2[m_Integer, d_Integer, a_String] :=ExternalCall[cddml2,CallPacket[0,{m,d,a}]]

AllVerticesWithAdjacency2[m_Integer, d_Integer, a_String] := ExternalCall[cddmlgmp,CallPacket[1,{m,d,a}]]
*)

(* AllVertices[m_Integer, d_Integer, a_String] := ExternalCall[cddml,CallPacket[0,{m,d,a}]] *)

		
		
CddVerticesCore[game_,opts___?OptionQ] := Module[{ratexact,liste,mat,values,extmat,string1,m,d1,vert,linearity,incidences,sortlist,sortlistc},
        Clear[liste];Clear[mat];Clear[values];Clear[extmat];
        ratexact = RationalExact /. {opts} /.Options[CddVerticesCore]; 
	liste = Table[If[i>=j,1,0],{i,Length[T]},{j,Length[T]}];
	mat = Flatten[Table[Permutations[liste[[i]]],{i,Length[liste]}],1];
        values = v[#]& /@ Rest[Coalitions];
	extmat=MapThread[Prepend,{mat,-values}];
        string1=ToString[extmat,InputForm];
	{m,d1}=Dimensions[extmat];
	{{vert,linearity},incidences}= Which[ratexact==False,AllVertices[m,d1,Flatten[N[extmat]]],
                                             True,AllVertices[m,d1,string1]];
	sortlist=Which[ratexact==False,Drop[Sort[vert],Length[T]],True,Drop[Sort[ToExpression[vert]],Length[T]]];
        sortlistc = Chop[sortlist,10^(-12)];
	Table[Drop[sortlistc[[i]],1],{i,Length[sortlistc]}]
		]


CddVerticesImputationSet[game_,opts___?OptionQ] := 
  Module[{ratexact,liste, zwmat, finmat, bvec, extmat, string1, m, d1, vert, linearity, 
      incidences, sortlist},
    ratexact =  RationalExact /. {opts} /. Options[CddVerticesReasonableSet];
    liste = Table[If[i >= j, 1, 0], {i, Length[T]}, {j, Length[T]}];
    zwmat = Flatten[Table[Permutations[liste[[i]]], {i, Length[liste]}], 1];
    finmat = Append[Take[zwmat, Length[T]], Last[zwmat]];
    bvec = Append[v[{#}]&/@T, v[T]];
    extmat = MapThread[Prepend, {finmat, -bvec}];
    string1=ToString[extmat,InputForm];
   {m, d1} = Dimensions[extmat];
   {{vert, linearity}, incidences} = Which[ratexact===False, AllVertices[m, d1, Flatten[N[extmat]]],True,AllVertices[m, d1, string1]];
    sortlist = Which[ratexact===False,Drop[Sort[vert], Length[T]],True,Drop[Sort[ToExpression[vert]], Length[T]]];
   Table[Drop[sortlist[[i]], 1], {i, Length[sortlist]}]
    ]

CddVerticesReasonableSet[game_,opts___?OptionQ] := 
  Module[{propcont,ratexact,liste, zwmat, finmat, bvec, extmat, string1, m, d1, vert, linearity, 
      incidences, sortlist},
    ratexact =  RationalExact /. {opts} /. Options[CddVerticesReasonableSet];
    propcont =  ProperContribution /. {opts} /. Options[CddVerticesReasonableSet];
    liste = Table[If[i >= j, 1, 0], {i, Length[T]}, {j, Length[T]}];
    zwmat = Flatten[Table[Permutations[liste[[i]]], {i, Length[liste]}], 1];
    finmat = -Append[Take[zwmat, Length[T]], Last[zwmat]];
    bvec = Which[propcont===False,Rationalize[Append[ReasonableOutcome[game], v[T]]],
           True,Rationalize[Append[ProperAmount[game], v[T]]]];
    extmat = MapThread[Prepend, {finmat, bvec}];
   {m, d1} = Dimensions[extmat];
    string1=ToString[extmat,InputForm];
   {{vert, linearity}, incidences} = Which[ratexact===False, AllVertices[m, d1, Flatten[N[extmat]]],True,AllVertices[m, d1, string1]];
    sortlist = Which[ratexact===False,Drop[Sort[vert], Length[T]],True,Drop[Sort[ToExpression[vert]], Length[T]]];
    Which[Length[sortlist]===1,Print["Upper Set is an unique point."],True,True];
   Table[Drop[sortlist[[i]], 1], {i, Length[sortlist]}]
    ]


CddVerticesLowerSet[game_,opts___?OptionQ] := 
  Module[{ratexact,liste, zwmat, finmat, bvec, smc, extmat, string1, m, d1, vert, linearity, 
      incidences, sortlist},
    ratexact =  RationalExact /. {opts} /. Options[CddVerticesLowerSet];
    liste = Table[If[i >= j, 1, 0], {i, Length[T]}, {j, Length[T]}];
    zwmat = Flatten[Table[Permutations[liste[[i]]], {i, Length[liste]}], 1];
    finmat = -Append[Take[zwmat, Length[T]], Last[zwmat]];
    smc = SmallestContributionVector[game];
    bvec = Rationalize[Append[smc, -v[T]]];
    extmat = MapThread[Prepend, {-finmat, bvec}];
   {m, d1} = Dimensions[extmat];
    string1=ToString[extmat,InputForm];
   {{vert, linearity}, incidences} = Which[ratexact===False, AllVertices[m, d1, Flatten[N[extmat]]],True,AllVertices[m, d1, string1]];
    sortlist = Which[ratexact===False,Drop[Sort[vert], Length[T]],True,Drop[Sort[ToExpression[vert]], Length[T]]];
    Which[Length[sortlist]===1,Print["Lower Set is an unique point."];,True,True];
   Table[Drop[sortlist[[i]], 1], {i, Length[sortlist]}]
    ]



(* Under Unix uncoment all commands related to CddGmp *) 
(*


CddGmpVerticesCore[game_,opts___?OptionQ] := Module[{ratexact,withinc,liste,mat,values,extmat,string1,string2,m,d1,vert,vertlist,linearity,incidences,sortlist},
        Clear[liste];Clear[mat];Clear[values];Clear[extmat];
        ratexact = RationalExact /. {opts} /.Options[CddGmpVerticesCore]; 
        withinc = WithIncidences /. {opts} /.Options[CddGmpVerticesCore]; 
	liste = Table[If[i>=j,1,0],{i,Length[T]},{j,Length[T]}];
	mat = Flatten[Table[Permutations[liste[[i]]],{i,Length[liste]}],1];
        values = Rationalize[v[#]& /@ Rest[Coalitions]];
	extmat=MapThread[Prepend,{mat,-values}];
        string1=ToString[extmat,InputForm];
        string2=ToString[N[extmat],InputForm];
	{m,d1}=Dimensions[extmat];
	{{vert,linearity},incidences}= If[ratexact==True, AllVertices[m,d1,string1],AllVertices2[m,d1,string2]];
        vert = Drop[Sort[vert],Length[T]];
        vertlist = Map[Drop[#,1]&, ToExpression[vert]];
        Which[(withinc == False) && (ratexact == False), Chop[vertlist,10^(-12)],
              (withinc == False) && (ratexact == True),  vertlist,
              (withinc == True) && (ratexact == False), {Chop[vertlist,10^(-12)], incidences},
              (withinc == True) && (ratexact == True), {vertlist,incidences},
              True, vertlist
              ]
	]




CddGmpPlotCore[game_,opts___?OptionQ] := 
   Module[{ratexact,liste,mat,values,extmat,string1,string2,m,d1,vert,vertlist,linearity,incidences,sortlist,ecdlist,eadlist,icdlist,iadlist},
        Clear[liste];Clear[mat];Clear[values];Clear[extmat];
        ratexact = RationalExact /. {opts} /.Options[CddGmpPlotCore]; 
	liste = Table[If[i>=j,1,0],{i,Length[T]},{j,Length[T]}];
	mat = Flatten[Table[Permutations[liste[[i]]],{i,Length[liste]}],1];
        values = Rationalize[v[#]& /@ Rest[Coalitions]];
	extmat=MapThread[Prepend,{mat,-values}];
        string1=ToString[extmat,InputForm];
        string2=ToString[N[extmat],InputForm];
	{m,d1}=Dimensions[extmat];
	 {{vert,linearity},ecdlist,eadlist,icdlist,iadlist} = AllVerticesWithAdjacency2[m,d1,string1];
        vertlist = Map[Drop[#,1]&, ToExpression[vert]];
        {{vertlist,linearity},ecdlist,eadlist,icdlist,iadlist,extmat}
 ]




CddGmpPlotImputationSet[game_] := 
  Module[{liste, zwmat, finmat, bvec, extmat, m, d1, vert, string1, string2, vertlist, linearity, 
      incidences, sortlist, ecdlist,eadlist,icdlist,iadlist},
    liste = Table[If[i >= j, 1, 0], {i, Length[T]}, {j, Length[T]}];
    zwmat = Flatten[Table[Permutations[liste[[i]]], {i, Length[liste]}], 1];
    finmat = Append[Take[zwmat, Length[T]], Last[zwmat]];
    bvec = Rationalize[Append[v[{#}]&/@T, v[T]]];
    extmat = MapThread[Prepend, {finmat, -bvec}];
    {m, d1} = Dimensions[extmat];
    string1=ToString[extmat,InputForm];
    string2=ToString[N[extmat],InputForm];
    {{vert,linearity},ecdlist,eadlist,icdlist,iadlist} = AllVerticesWithAdjacency2[m,d1,string1];
    vert = Drop[Sort[vert],Length[T]];
    vertlist = Map[Drop[#,1]&, ToExpression[vert]];
    {{vertlist,linearity},ecdlist,eadlist,icdlist,iadlist,extmat}
    ]



CddGmpPlotReasonableSet[game_,opts___?OptionQ] := 
  Module[{propcont,liste, zwmat, finmat, reasout,propout,bvec, extmat, m, d1, vert, vertlist, string1, string2, linearity, 
      incidences, sortlist},
    propcont =  ProperContribution /. {opts} /. Options[CddGmpPlotReasonableSet];
    liste = Table[If[i >= j, 1, 0], {i, Length[T]}, {j, Length[T]}];
    zwmat = Flatten[Table[Permutations[liste[[i]]], {i, Length[liste]}], 1];
    finmat = -Append[Take[zwmat, Length[T]], Last[zwmat]];
    reasout = ReasonableOutcome[game];
    propout = ProperAmount[game];
    Which[(reasout===propout)===True,Print["Reasonable set coincides with the Upper set!"],True,True];
    bvec = Which[propcont===False,Rationalize[Append[reasout, v[T]]],
                 True,Rationalize[Append[propout, v[T]]]];
    extmat = MapThread[Prepend, {finmat, bvec}];
   {m, d1} = Dimensions[extmat];
    string1=ToString[extmat,InputForm];
    string2=ToString[N[extmat],InputForm];
    {{vert,linearity},ecdlist,eadlist,icdlist,iadlist} = AllVerticesWithAdjacency2[m,d1,string1];
    vert = Drop[Sort[vert],Length[T]];
    vertlist = Map[Drop[#,1]&, ToExpression[vert]];
    Which[Length[vertlist]===1,Print["Upper Set is an unique point."];,True,True];
    {{vertlist,linearity},ecdlist,eadlist,icdlist,iadlist,extmat}
    ]



CddGmpPlotLowerSet[game_,opts___?OptionQ] := 
  Module[{liste, zwmat, finmat, bvec, smc, extmat, m, d1, vert, vertlist, string1, string2, linearity, 
      incidences, sortlist},
    liste = Table[If[i >= j, 1, 0], {i, Length[T]}, {j, Length[T]}];
    zwmat = Flatten[Table[Permutations[liste[[i]]], {i, Length[liste]}], 1];
    finmat = -Append[Take[zwmat, Length[T]], Last[zwmat]];
    smc = SmallestContributionVector[game];
    Which[(smc === Table[0,{i,Length[T]}])===True, Print["The Lower Set coincides with the Imputation Set!"],True,True];
    bvec = Rationalize[Append[smc, -v[T]]];
    extmat = MapThread[Prepend, {-finmat, bvec}];
   {m, d1} = Dimensions[extmat];
    string1=ToString[extmat,InputForm];
    string2=ToString[N[extmat],InputForm];
    {{vert,linearity},ecdlist,eadlist,icdlist,iadlist} = AllVerticesWithAdjacency2[m,d1,string1];
    vert = Drop[Sort[vert],Length[T]];
    vertlist = Map[Drop[#,1]&, ToExpression[vert]];
    Which[Length[vertlist]===1,Print["Lower Set is an unique point."];,True,True];
    {{vertlist,linearity},ecdlist,eadlist,icdlist,iadlist,extmat}
    ]

FilledCore[game_, payoff_:{}, opts___?OptionQ] := 
  Module[{vert, chvert, posvert, extvert, vertToSimplex, apptovert, 
      trianglepts, poly, figure, drawSimp},
    vert = CddGmpVerticesCore[game];
    chvert = BelongToCoreQ[game, vert];
    posvert = Position[chvert, True];
    extvert = Sort[Extract[vert, posvert]];
    vertToSimplex = Vec3DToSimplex[#] & /@ extvert;
    apptovert = 
      FlattenAt[Append[vertToSimplex, Take[vertToSimplex, 2]], 
        Length[vertToSimplex] + 1];
     trianglepts = 
      Table[Part[apptovert, {i, i + 1, i + 2}], {i, 1, 
          Length[vertToSimplex]}];
    poly = Polygon[#] & /@ trianglepts;
    figure = Graphics[{GrayLevel[.8], poly}];
    drawSimp = 
      DrawSimplex[{Core[game], payoff}, DisplayFunction -> Identity];
    Show[figure, drawSimp, PlotRange -> All]
    ]

*)

Vec3DToSimplex[{x1_?NumberQ, x2_?NumberQ, x3_?NumberQ}] := 
  Module[{}, {(x2 - x1) Sqrt[3]/2, x3 - (x1 + x2)/2}]

(* End mathlink section *)


(* section adjusted worth vector starts *)

AdjustedWorthVectors[game_, k_] :=
  Module[{permliste},
   If[Depth[k]==1,
    permliste = Permutations[T];
    Table[
      Table[If[SubCall1[permliste[[h]], i, k], 
          Map[v, PlayerIJoin[permliste[[h]], i]][[1]] - 
            Map[v, PDeltaI[permliste[[h]], i]][[1]],
          If[SubCall2[permliste[[h]], i, k], UtopiaPayoff[game, i],
              v[T] - Map[v, PDeltaI[permliste[[h]], i]][[1]] - 
              UtopiaSum[game, DeltaJSet[permliste[[h]], k]]]
             ],
               {i, Length[T]}], {h, Length[permliste]}], 
         DisplayRem[k]
    ]
   ]


DisplayRem[k_]:=(
         Print["Depth is equal to ",Depth[k]];
         Print["Usage: AdjustedWorthVectors[game,k] and kCover[game,k]"];
         Print["The varibale 'k' has not the correct input format."];
         Print["The variable 'k' must be a natural number and not a list."]);

SubCall1[permele_, player_, k_] := permele[[player]] < k
SubCall2[permele_, player_, k_] := permele[[player]] > k

PDeltaI[list_, player_] := Module[{thresh, newlist, genlist = {}, retval},
    thresh = list[[player]];
    newlist = DeleteCases[list, list[[player]]];
    retval = Table[
        If[newlist[[j]] < thresh, Append[genlist, newlist[[j]]], {}
          ], {j, Length[newlist]}] ;
    List[Sort[Flatten[retval]]]
    ]

PlayerIJoin[list_, player_] := 
  List[Sort[Flatten[Append[PDeltaI[list, player], player]]]]

DeltaJSet[list_, k_] := Module[{genlist = {}, pos, retval},
    pos = Flatten[
        Table[If[list[[i]] > k, Append[genlist, list[[i]]], {}], {i, 
            Length[list]}]];
    retval = Sort[
        Flatten[Table[Position[list, pos[[i]]], {i, Length[pos]}]]];
    retval
    ]

(* section adjusted worth vector ends *)


(* section general k-convex functions starts *)

Nuc1convex[game_]:= UtopiaVector[game] - Extract[Gap[game],-1] / Length[T]

UtopiaPayoff[i_]:= v[T] - v[DeleteCases[T,i]]

UtopiaPayoff[game_, i_]:= v[T] - v[DeleteCases[T,i]] 

(* bavect[game_]:= Table[UtopiaPayoff[i],{i,Length[T]}] *)

UtopiaSum[game_, S_List]:=  Plus  @@ UtopiaPayoff /@ S 

UtopiaVector[game_]:= UtopiaPayoff[game,#] & /@ T

Gap[game_]:= (UtopiaSum[game,#] - v[#]) & /@ Coalitions 


vkS[game_,S_]:= UtopiaSum[game,S] - Take[Gap[game],-1] 

kCover[game_,k_]:= If[Depth[k]==1,
       Flatten[If[Length[#] < k, v[#], vkS[game,#]] & /@ Coalitions],
       DisplayRem[k]
  ]
 

AdjustedEffVector[game_,i_]:= UtopiaVector[game] - Last[Gap[game]] Extract[Permutations[
                      Prepend[Table[0 j,{j,Length[T]-1}],1]],i
] 

AdjustedEffUpperVectors[game_]:=Table[AdjustedEffVector[game,i],{i,Length[T]}]



Concession[game_]:= Map[Min,Table[(UtopiaSum[game,#] - v[#]) &  /@ \
W[i],{i,Length[T]}]]

DisagreeConvex[game_]:=Table[UtopiaPayoff[i]-Extract[Gap[game],i+1],{i,Length[T]}]

Disagreement[game_]:= UtopiaVector[game] - Concession[game]

TauVal[game_]:= UtopiaVector[game] - (Extract[Gap[game],-1] Concession[game])/ \
Apply[Plus,Concession[game]] 

TauValue[game_]:=
		If[Last[Gap[game]]==0,Tau[game],TauVal[game]];
		Tau[game_]:= UtopiaVector[game]

(* section general k-convex functions starts *)




(* section miscellaneously functions starts *)


MargValue[game_]:=Module[{perm,list},
  list = T;
  perm = Permutations[list];
  Table[MapThread[
      Subtract,{Map[v,pred[perm[[i]]]],Map[v,mengen[perm[[i]]]]}],{i,
      Length[perm]}]
]


pred[list_]:= Module[{post,vorg,ober},
		post= Flatten[Table[Flatten[Position[list,i]],{i,Length[T]}]];
		vorg= Table[Take[list,post[[i]]-1],{i,Length[T]}];
	Table[Sort[Extract[FlattenAt[MapThread[List,{vorg,T}],{i,1}],i]],{i,
        Length[T]}]
		]

mengen[list_]:= 
  Module[{post},post= \
    Flatten[Table[Flatten[Position[list,i]],{i,Length[T]}]];
    Table[Sort[Take[list,post[[i]]-1]],{i,Length[T]}]
	]



CoreElementsQ[game_,payoffs_]:= Module[{},
  If[Depth[payoffs] == 3,
    Table[InCoreQ[payoffs[[i]],game],{i,Length[payoffs]}],
     If[Depth[payoffs] == 2, InCoreQ[payoffs,game],
       DisplayUsageRem[payoffs]
    ]
   ]
  ]

(*
An imputation x is in the core iff x =< dua v.
*)

BelongToCoreQ[game_, payoffs_] := Module[{},
    If[Depth[payoffs] == 3, PayList[game, payoffs],  
      If[Depth[payoffs] == 2, PaySole[game, payoffs],
       DisplayUsageRem[payoffs]
       ]
     ]
    ]

DisplayUsageRem[payoffs_] := (      
       Print["Depth is equal to ",Depth[payoffs]];
       Print["Usage: BelongToCoreQ[game,payoffs] and CoreElementsQ[game,payoffs]"];
       Print["Input format of the variable 'payoffs' is not correct."];
       Print["The variable 'payoffs' must be a list of payoff vectors or a single payoff vector."]);


PayList[game_, list_] := 
  Module[{totsumS, imp, assalloc, ducoal, ins, trueli, wahrQ},
    totsumS := x[#] & /@ Coalitions; 
    imp = x[#] & /@ T;
    assalloc = MapThread[Rule, {imp, #}] & /@  list;
    ins = totsumS /. assalloc // Rationalize;
    ducoal = DualGame[game];
    trueli = MapThread[LessEqual, {#, ducoal}] & /@ ins;
    wahrQ = Apply[And, #] & /@ trueli;
    wahrQ
    ]



PaySole[game_, payoff_] := 
  Module[{totsumS, imp, assalloc, ducoal, ins, trueli, wahrQ},
    totsumS := x[#] & /@ Coalitions; 
    imp = x[#] & /@ T;
    assalloc = MapThread[Rule, {imp, payoff}];
    ins = totsumS /. assalloc // Rationalize;
    ducoal = DualGame[game];
    trueli = MapThread[LessEqual, {ins, ducoal}];
    wahrQ = Apply[And, trueli] ;
    wahrQ
    ]

exzess:=(v[#]-x[#])& /@ Coalitions

CoordS[game_,S_List]:=Module[{liste,upper,iterat,mult,koeffient,value},
	liste=(Length[#])& /@ Subsets[S];
	upper=Table[Length[S],{Length[liste]}];
	iterat=MapThread[Subtract,{upper,liste}];
	mult= Table[-1,{Length[liste]}];
	koeffient=MapThread[Power,{mult,iterat}];
	value=(v[#])& /@ Subsets[S];
	Apply[Plus,MapThread[Times,{koeffient,value}]]
]


UnanimityCoordinates[game_]:=(CoordS[game,#])& /@ Coalitions 

MinUnanimityCoordinates[game_]:=Module[{anz,sepset},
	anz=(Length[#])& /@ Subsets[T];
	sepset=Table[Extract[Subsets[T],Position[anz,i]],{i,Length[T]}];
	Table[Min[(CoordS[game,#])& /@ sepset[[i]]],{i,1,Length[T]}]
		]

StrictlyConvexUnanConditionQ[game_]:= Module[{unanc},
  unanc=EvalSumMinCoord[game];
  Apply[And,Positive[unanc]]
]

ConvexUnanConditionQ[game_]:= Module[{unanc},
  unanc=EvalSumMinCoord[game];
  Apply[And,NonNegative[unanc]]
]



(* Cor. 2.1 in C. Rafels and N. Ybern (1995) *)

EvalSumMinCoord[game_]:=
  Module[{smallestcoord,alphak,post,firstneg,s0,alltrueQ,fac,retval,singleQ},
    smallestcoord=MinUnanimityCoordinates[game];
    alphak=Drop[smallestcoord,1];
    post=NonNegative[#]&/@ alphak;
    alltrueQ=Apply[And,post];
    If[alltrueQ \[Equal] False,
      firstneg=Take[Flatten[Position[post,False]],1][[1]];
      s0=firstneg + 1;
      If[s0>Length[T]-2,
        DisplaySize[game,s0];
        smallestcoord,
        restmin=Table[Take[alphak,i],{i,Length[alphak]}];
        fac=Table[Table[(k!)/((j!)(k-j)!),{j,0,k}],{k,0,Length[T]-2}];
        retval=Apply[Plus,#]&/@MapThread[Times,{fac,restmin}];
        retval],
          singleQ=NonNegative[Take[smallestcoord,1]];
          alltrueQ=Apply[And,{alltrueQ,singleQ}][[1]];
          If[alltrueQ == True,
            PrintMessageFirst;smallestcoord,
               PrintMessageSec;smallestcoord] 
         ] 
       ]
     

DisplaySize[game_, s0_] := (
      Print[
        "Minimum coalition size with negative unanimity coordinate is equal \
to ", s0];
      Print["Coalition size of T-2 is ", Length[T] - 2];
      Print[s0, " greater than ", Length[T] - 2];
      Print["No sum of minimum unanimity coordinates can be evaluated."];
      Print["The minimum unanimity coordinates are:"]
      );

PrintMessageFirst:=(
      Print["No negative unanimity coordinates found."];
      Print["Game is convex."];
      Print["The minimum unanimity coordinates are:"]);

PrintMessageSec:=(
      Print["No negative unanimity coordinates found."];
      Print["The minimum unanimity coordinates are:"]);

End[]

Protect[Global`Co,coal]
Protect[Global`sij]

(*

Protect[DefineGame]
Protect[DualGame]
Protect[SuperAdditiveQ]
Protect[WeaklySuperAdditiveQ]
Protect[Assign]
Protect[SelectSuperSets]
Protect[IncreasingMargContributions]
Protect[ConvCheck]
Protect[ConvexQ]
Protect[StrIncreasMargContrib]
Protect[ConvStrQ]
Protect[ConvexStrQ]
Protect[AvConvexQ]
Protect[AvSumCheck]
Protect[GameMonotoneQ]
Protect[MonotoneQ]
Protect[ZeroMonotoneQ]
Protect[OneNormalization]
Protect[ZeroNormalization]
Protect[ZeroOneNormalization]
Protect[ContributionSum]
Protect[SmallestContributionVector]
Protect[SmallestContribution]
Protect[Scrb]
Protect[ScrbSolution]
Protect[ExcessPayoff]
Protect[MaxExcessBalanced]
Protect[MaxExcessSets]
Protect[IntersectionOfMaxExcessSets]
Protect[KernelCalculation]
Protect[Kernel]
Protect[KernelVertices]
Protect[ModifiedKernel]
Protect[ModifiedNucleolus]
Protect[LexiCenter]
Protect[Prenucleolus]
Protect[FindKernelSolution]
Protect[FindPreKernelSolution]
Protect[ImputationQ]
Protect[KernelImputationQ]
Protect[KernelImputationListQ]
Protect[PreKernelQ]
Protect[Transfer]
Protect[DeltaLP]
Protect[TransferConstraints]
Protect[AllConstraints]
Protect[MaxSurplus]
Protect[AllMaxSurpluses]
Protect[EpsValue]
Protect[EpsCore]
Protect[FirstCriticalVal]
Protect[SecondCriticalVal]
Protect[ThirdCriticalVal]
Protect[FourthCriticalVal]
Protect[FifthCriticalVal]
Protect[StarCriticalVal]
Protect[SecondStarCriticalVal]
Protect[ThirdStarCriticalVal]
Protect[LargestAmount]
Protect[ProperAmount]
Protect[ReasonableSet]
Protect[ReasonableOutcome]
Protect[LowerSetQ]
Protect[UpperSetQ]
Protect[LowerSetIncImputationQ]
Protect[UpperSetIncImputationQ]
Protect[IntersectionUpperLowerSetQ]
Protect[GrandCoalitionLargestValueQ]
Protect[PreKernelEqualsKernelQ]
Protect[Quota]
Protect[VerticesCore]
Protect[CddVerticesCore]
Protect[CddGmpVerticesCore]
Protect[CddGmpPlotCore]
Protect[CddVerticesImputationSet]
Protect[CddGmpPlotImputationSet]
Protect[CddVerticesReasonableSet]
Protect[CddGmpPlotReasonableSet]
Protect[CddVerticesLowerSet]
Protect[CddGmpPlotLowerSet]
Protect[FilledCore]
Protect[Nuc1convex]
Protect[NewShapley]
Protect[UtopiaVector]
Protect[UtopiaPayoff]
Protect[UtopiaSum]
Protect[Gap]
Protect[AdjustedEffVector]
Protect[AdjustedEffUpperVectors]
Protect[AdjustedWorthVectors]
Protect[Concession]
Protect[DisagreeConvex]
Protect[Disagreement]
Protect[TauValue]
Protect[MargValue]
Protect[kCover]
Protect[CoreElementsQ]
Protect[BelongToCoreQ]
Protect[CoordS]
Protect[UnanimityCoordinates]
Protect[MinUnanimityCoordinates]
Protect[StrictlyConvexUnanConditionQ]
Protect[ConvexUnanConditionQ]
Protect[EvalSumMinCoord]

*)




Print["==================================================="];
Print["Package 'TuGames' loaded"];
Print["==================================================="];

EndPackage[]









