(* :Title: TuGamesAux.m *)
(* Release Date: 27.07.05 *)

(* :Context: TuGamesAux` *)

(* :Summary:	
    This package provides some auxiliary functions to the package TuGames.m.
*)			

(* :Author:		
    Holger Ingmar Meinhardt
    Department of Economics
    University of Karlsruhe 
    hme@vwl3.wiwi.uni-karlsruhe.de			
*)

(* :Package Version: Preliminary *)

(* 
   :Mathematica Version: 3.0, 4.0, 4.1, 4.2, 5.0 

*)

(* :Sources:
   Maschler JET, 13, pp. 184-192.

*)

(*:Keywords:
 Extracting the coefficient from a LP. Solving the primal and dual problem.
*)




BeginPackage["TuGamesAux`"]


ComplementaryMarket::usage = 
"ComplementaryMarket[Tlist,Plist_,Qlist,opts] computes the coalitional values of \n
a market situation with complementary goods. See Maschler JET, 13, pp. 184-192. The set P \n
can be understood as manufactures and Q as workers. The set T is the player set with P union Q."

GreedyBankruptcy::usage = 
"GreedyBankruptcy[T,Estate,claims] computes the coalitional values 
 for a greedy bankruptcy game."


LPtoMatrix::usage = 
"LPtoMatrix[zf, const, var] extracts the coefficient from a LP. Where zf is the objective function \n
const is the linear constraint set and var is set of variables.";


ModestBankruptcy::usage = 
"ModestBankruptcy[T,Estate,claims] computes the coalitional values 
 for a modest bankruptcy game."


SolvePrimal::usage = 
"SolvePrimal[zf, const, var] solves a primal problem."


SolveDual::usage = 
"SolveDual[zf, const, var] solves the corresponding dual problem."

EuclidianDistance::usage = 
"EuclidianDistance[liste,refliste] computes the Euclidian distance\n
of the vector 'liste' w.r.t. the vector 'refliste'."


NearRingQ::usage =
"NearRingQ[list,T] checks if the collection of sets is near ring."

SelectPart::usage=
""

AssignSymVal::usage = 
""


Options[ComplementaryMarket] = {MarketParameter -> (1/2)}


Begin["`Private`"]

(*Extracting the coefficients from an LP *)

LPtoMatrix[obf_,eq_List,var_List]:=
    Module[{zf, coemat, coebv, greq, lseq, eqeq, grout, lsout, eqout, ext1,
       ext2, ext3, bv01, bv02, bv03, nbv01, bvec},
    zf = Coefficient[obf, var];
    coemat = Coefficient[#[[1]], var] & /@ eq;
    coebv = Last[#] & /@ eq;
    greq = Position[eq, GreaterEqual];
    eqeq = Position[eq, Equal];
    lseq = Position[eq, LessEqual];
    grout = Outer[List, First[#] & /@ greq];
    eqout = Outer[List, First[#] & /@ eqeq];
    lsout = Outer[List, First[#] & /@ lseq];
    ext1 = If[Length[grout] == 0, grout, Extract[coebv, grout]];
    ext2 = If[Length[eqout] == 0, eqout, Extract[coebv, eqout]];
    ext3 = If[Length[lsout] == 0, lsout, Extract[coebv, lsout]];
    bv01 = If[Length[ext1] == 0, ext1,
        Flatten[MapThread[List, {{#}, {1}}]] & /@ ext1];
    bv02 = 
      If[Length[ext2] == 0, ext2, 
        Flatten[MapThread[List, {{#}, {0}}]] & /@ ext2];
    bv03 = 
      If[Length[ext3] == 0, ext3, 
        Flatten[MapThread[List, {{#}, {-1}}]] & /@ ext3];
    bv01 = If[Length[bv01] == 0, {bv01}, bv01 ];
    nbv01 = If[Length[bv02] == 0, bv01, AppendTo[bv01, bv02]; FlattenAt[bv01,Length[bv01]]];
    nbv01 = If[Length[bv03] == 0, nbv01, AppendTo[nbv01, bv03]; FlattenAt[nbv01,Length[nbv01]]];
    bvec = DeleteCases[nbv01,{}];
    {zf, coemat, bvec}
    ]


SolvePrimal[zf_,eq_List,var_List,opts___?OptionQ] := 
  Module[{obf, cmat, bvect, res, nzf, rl},
    {obf, cmat, bvect} = LPtoMatrix[zf, Simplify[eq], var];
    res = LinearProgramming[obf, cmat, bvect, opts];
    nzf = obf.res;
    rl = MapThread[Rule, {var, res}];
    Prepend[{rl}, nzf]
    ]


SolveDual[zf_,eq_List,var_List, opts___?OptionQ]:=
 Module[{obf, cmat, bvect, tobf, tmat, tbv, res, nzf, yvar, rl},
   {obf, cmat, bvect} = LPtoMatrix[zf,eq, var];
   tobf = First[#] & /@ bvect;
   tmat = Transpose[cmat];
   tbv = MapThread[List, {obf, -obf}];
   res = LinearProgramming[-tobf, tmat, tbv, opts];
   dim = Dimensions[tmat];
   yvar = Global`y[#] & /@ Table[i, {i, Last[dim]}];
   nzf = tobf.res;
   rl = MapThread[Rule, {yvar, res}];
   Prepend[{rl}, nzf]
  ]


(* Near Ring *)

NearRingQ[set_List,T1_] := Module[{boolval},
    boolval = Which[Depth[set] === 3,  CheckNearRing[set,set,T1],
                    Depth[set] === 4,  CheckNearRing[#,#,T1] & /@ set,
                    True, DisplayMessNearRing[set]];
   Return[boolval]
    ]


CheckNearRing[Univers_List, RestSet_List, T2_, prevres_:{}] := 
  Module[{fi, restuni, res, allres, boolval},
    fi = Flatten[Take[RestSet, 1]];
    res = NearRingDef[fi, #, T2, Univers] & /@ RestSet;
    restuni = Delete[RestSet, 1];
    allres = FlattenAt[Append[res, prevres], Length[res] + 1];
    Which[Length[restuni] === 0, Apply[And, allres], True, 
      CheckNearRing[Univers, restuni, T2, allres]]
    ]


NearRingDef[A_, B_, R1_,Univers_List] := Module[{},
    Union[A, B] === R1 || Intersection[A, B] === {} || (Length[Position[Univers, Union[A, B]]] != 0 && 
          Length[Position[Univers, Intersection[A, B]]] != 0)
    
    ]


DisplayMessNearRing[set_List] := (Print["Depth is equal to ", Depth[set]];
      Print["The collection of sets must be of depth 3 or 4."];
      Print["Usage: NearRi[set]"]);


(* Market Situation  *)

teilm[T_List]:=Sort[Flatten /@ Distribute[({{}, {#1}} &) /@ T, List]];

ComplementaryMarket[T_List,P_List,Q_List,opts___?OptionQ]:= Module[{em,param,coal,val},
    em = Intersection[P,Q];
    Which[Length[em] != 0, Print["The sets P and Q are not disjoint."]; Break[],
          True, 
              param = MarketParameter /. {opts} /. Options[ComplementaryMarket];  
              coal = teilm[T];
              val = Min[Length[Intersection[#, P]], a Length[Intersection[#, Q]]] & /@ coal;
              val /. a -> param
          ]
]

(* Bankruptcy Situation *)

x[S_List]:= Plus @@ x /@ S

ModestBankruptcy[T_List,E_,claims_List]:= Module[{asscl,compset,vecval},
    asscl = MapThread[Rule,{x /@ T,claims}];
    compset = Complement[T,#] &/@ teilm[T];
    vecval = Max[0, E -  x[#]] &/@ compset; 
    vecval /.asscl
]  

GreedyBankruptcy[T_List,E_,claims_List]:= Module[{asscl,vecval},
    asscl = MapThread[Rule,{x /@ T,claims}];
    vecval = Min[E, x[#]] &/@ teilm[T];
    vecval /.asscl
]  

EuclidianDistance[liste_,refliste_]:=
          Which[Depth[liste] == 3, EuclidianDist[#, refliste] & /@ liste,
                Depth[liste] == 2, EuclidianDist[liste, refliste],
                True,DisplayEuk[liste]]

EuclidianDist[liste_, refliste_] := N[Power[Apply[Plus, MapThread[Power[#1 - #2, 2] &, {liste, refliste}]], (1/2)]]

DisplayEuk[liste_]:=(
       Print["Depth is equal to ",Depth[liste]];
       Print["Usage: EuclidianDistance[liste,refliste]"];
       Print["Input format of the variable 'liste' is not correct."];
       Print["The variable 'liste' must be a list."]
);


(* Assigning symmetric values *)

SelectPart[part1_List, part2_List] := 
  Module[{lng, posof1, posof3, extpt1of1, extpt2of1, extpt1of3, extpt2of3, 
      pt1of1, pt2of1, newpart1, newpart2, mng},
    Which[Length[part1] === Length[part2],
      lng = Length[#] & /@ part1;
      posof1 = Position[lng, 1];
      posof3 = Position[lng, 3];
      {extpt1of1, extpt2of1} = Extract[#, posof1] & /@ {part1, part2};
      {extpt1of3, extpt2of3} = Extract[#, posof3] & /@ {part1, part2};
      pt1of1 = Table[#, {i, Length[extpt1of1]}] & /@ extpt1of1;
      pt2of1 = Table[#, {i, Length[extpt2of1]}] & /@ extpt2of1;
      newpart1 = MapThread[Union[#1, #2] &, {extpt1of3, #}] & /@ pt2of1;
      newpart2 = MapThread[Union[#1, #2] &, {extpt2of3, #}] & /@ pt1of1;
       AppendTo[newpart1, #] & /@ newpart2;
      mng = First[newpart1];
      (AppendTo[mng, #] & /@ #1) & /@ Drop[newpart1, 1];
      Sort[mng],
      True, Print["The sets are not of equal length"]; Break[]]
    ]

AssignSymVal[univers_List, alp_List, storeval_:{}] := 
  Module[{newuni, newalp, compset, compval = storeval},
    Which[Length[univers] === 0, Return[compval]; Break[],
      True,
      compset = AssignValues[univers, alp];
      newalp = If[First[univers] === First[alp], Drop[alp, 1], alp];
      AppendTo[compval, compset];
      newuni = Drop[univers, 1];
      Which[Length[univers] != 0, AssignSymVal[newuni, newalp, compval], True,
         compval]]
    ]

AssignValues[univers_List, alp_List] := Module[{},
      Which[Length[alp] != 0,
        Which[First[univers] === First[alp], Global`\[Alpha], 
          Length[First[univers]] <= 4, Global`\[Lambda], True, Global`\[Sigma]],
                          True,
        Which[Length[First[univers]] ==  5,Global`\[Theta],
          Length[First[univers]] ==  6, Global`\[Phi],
          Length[First[univers]] >= 7, 
          If[Length[First[univers]] === 7, Global`\[Beta], 
            v[First[univers]] = Global`\[Gamma]], True, Global`\[Sigma]]]];


End[]

(*
Protect[ComplementaryMarket]
Protect[GreedyBankruptcy]
Protect[LPtoMatrix]
Protect[ModestBankruptcy]
Protect[SolvePrimal]
Protect[SolveDual]
Protect[EuclidianDistance]
*)


EndPackage[]
